"""
Memory Persistence Module - 记忆持久化模块
实现完整的数据库持久化功能，支持所有记忆类型的存储和检索

提供功能：
- SQLite数据库存储
- 优化的查询和索引
- 批量操作支持
- 数据迁移和备份
- 性能监控和优化
- 事务管理
- 数据完整性保证
"""

import asyncio
import sqlite3
import aiosqlite
import json
import pickle
import hashlib
import uuid
from typing import Dict, Any, List, Optional, Union, Tuple
from dataclasses import dataclass, asdict
from datetime import datetime, timedelta
from pathlib import Path
from enum import Enum
import logging
import numpy as np
from contextlib import asynccontextmanager
import os

from .memory import (
    MemoryItem, MemoryType, EpisodicMemory, SemanticMemory, ProceduralMemory,
    MemoryImportance
)

logger = logging.getLogger(__name__)


class PersistenceMode(Enum):
    """持久化模式"""
    IMMEDIATE = "immediate"    # 立即持久化
    BATCH = "batch"           # 批量持久化
    PERIODIC = "periodic"     # 定期持久化
    LAZY = "lazy"            # 懒惰持久化（仅在关闭时）


class StorageBackend(Enum):
    """存储后端"""
    SQLITE = "sqlite"
    POSTGRESQL = "postgresql"
    MYSQL = "mysql"
    MEMORY = "memory"


@dataclass
class PersistenceConfig:
    """持久化配置"""
    storage_backend: StorageBackend = StorageBackend.SQLITE
    database_path: str = "memory_database.db"
    persistence_mode: PersistenceMode = PersistenceMode.BATCH
    batch_size: int = 100
    batch_timeout: float = 30.0  # 秒
    max_connections: int = 10
    enable_wal: bool = True      # Write-Ahead Logging
    enable_foreign_keys: bool = True
    vacuum_interval: int = 3600  # 秒
    backup_interval: int = 86400 # 秒
    retention_days: int = 365    # 数据保留天数
    enable_compression: bool = True
    enable_encryption: bool = False
    encryption_key: Optional[str] = None


class MemoryDatabaseSchema:
    """记忆数据库模式定义"""
    
    # 主要表结构
    TABLES = {
        'memory_items': '''
            CREATE TABLE IF NOT EXISTS memory_items (
                id TEXT PRIMARY KEY,
                content TEXT NOT NULL,
                content_hash TEXT NOT NULL,
                memory_type TEXT NOT NULL,
                importance REAL NOT NULL,
                access_count INTEGER DEFAULT 0,
                last_accessed TIMESTAMP,
                created_at TIMESTAMP NOT NULL,
                updated_at TIMESTAMP,
                tags TEXT,  -- JSON array
                metadata TEXT,  -- JSON object
                embedding BLOB,  -- Serialized numpy array
                embedding_model TEXT,
                semantic_keywords TEXT,  -- JSON array
                is_deleted BOOLEAN DEFAULT FALSE,
                version INTEGER DEFAULT 1
            )
        ''',
        
        'episodic_memories': '''
            CREATE TABLE IF NOT EXISTS episodic_memories (
                id TEXT PRIMARY KEY,
                episode_id TEXT UNIQUE NOT NULL,
                event TEXT NOT NULL,
                context TEXT NOT NULL,  -- JSON object
                participants TEXT NOT NULL,  -- JSON array
                location TEXT,
                timestamp TIMESTAMP NOT NULL,
                emotional_valence REAL DEFAULT 0.0,
                importance REAL DEFAULT 0.5,
                related_episodes TEXT,  -- JSON array
                metadata TEXT,  -- JSON object
                created_at TIMESTAMP NOT NULL,
                updated_at TIMESTAMP,
                is_deleted BOOLEAN DEFAULT FALSE
            )
        ''',
        
        'semantic_memories': '''
            CREATE TABLE IF NOT EXISTS semantic_memories (
                id TEXT PRIMARY KEY,
                concept_id TEXT UNIQUE NOT NULL,
                concept TEXT NOT NULL,
                definition TEXT NOT NULL,
                properties TEXT,  -- JSON object
                relationships TEXT,  -- JSON object
                confidence REAL DEFAULT 1.0,
                source TEXT,
                created_at TIMESTAMP NOT NULL,
                updated_at TIMESTAMP,
                metadata TEXT,  -- JSON object
                is_deleted BOOLEAN DEFAULT FALSE
            )
        ''',
        
        'procedural_memories': '''
            CREATE TABLE IF NOT EXISTS procedural_memories (
                id TEXT PRIMARY KEY,
                procedure_id TEXT UNIQUE NOT NULL,
                name TEXT NOT NULL,
                description TEXT NOT NULL,
                steps TEXT NOT NULL,  -- JSON array
                preconditions TEXT,  -- JSON array
                postconditions TEXT,  -- JSON array
                success_rate REAL DEFAULT 1.0,
                usage_count INTEGER DEFAULT 0,
                last_used TIMESTAMP,
                created_at TIMESTAMP NOT NULL,
                updated_at TIMESTAMP,
                metadata TEXT,  -- JSON object
                is_deleted BOOLEAN DEFAULT FALSE
            )
        ''',
        
        'memory_relationships': '''
            CREATE TABLE IF NOT EXISTS memory_relationships (
                id TEXT PRIMARY KEY,
                source_id TEXT NOT NULL,
                target_id TEXT NOT NULL,
                relationship_type TEXT NOT NULL,
                strength REAL DEFAULT 1.0,
                direction TEXT DEFAULT 'bidirectional',  -- unidirectional, bidirectional
                metadata TEXT,  -- JSON object
                created_at TIMESTAMP NOT NULL,
                updated_at TIMESTAMP,
                FOREIGN KEY (source_id) REFERENCES memory_items (id),
                FOREIGN KEY (target_id) REFERENCES memory_items (id)
            )
        ''',
        
        'memory_snapshots': '''
            CREATE TABLE IF NOT EXISTS memory_snapshots (
                id TEXT PRIMARY KEY,
                name TEXT,
                snapshot_data TEXT NOT NULL,  -- JSON object
                compression_type TEXT DEFAULT 'none',
                size_bytes INTEGER,
                checksum TEXT,
                created_at TIMESTAMP NOT NULL,
                metadata TEXT  -- JSON object
            )
        ''',
        
        'memory_statistics': '''
            CREATE TABLE IF NOT EXISTS memory_statistics (
                id TEXT PRIMARY KEY,
                stat_type TEXT NOT NULL,
                stat_name TEXT NOT NULL,
                stat_value REAL NOT NULL,
                timestamp TIMESTAMP NOT NULL,
                metadata TEXT  -- JSON object
            )
        ''',
        
        'memory_operations_log': '''
            CREATE TABLE IF NOT EXISTS memory_operations_log (
                id TEXT PRIMARY KEY,
                operation_type TEXT NOT NULL,  -- INSERT, UPDATE, DELETE, QUERY
                table_name TEXT NOT NULL,
                record_id TEXT,
                operation_data TEXT,  -- JSON object
                execution_time REAL,  -- milliseconds
                success BOOLEAN NOT NULL,
                error_message TEXT,
                timestamp TIMESTAMP NOT NULL
            )
        '''
    }
    
    # 索引定义
    INDEXES = [
        'CREATE INDEX IF NOT EXISTS idx_memory_items_type ON memory_items (memory_type)',
        'CREATE INDEX IF NOT EXISTS idx_memory_items_importance ON memory_items (importance DESC)',
        'CREATE INDEX IF NOT EXISTS idx_memory_items_created_at ON memory_items (created_at DESC)',
        'CREATE INDEX IF NOT EXISTS idx_memory_items_last_accessed ON memory_items (last_accessed DESC)',
        'CREATE INDEX IF NOT EXISTS idx_memory_items_content_hash ON memory_items (content_hash)',
        'CREATE INDEX IF NOT EXISTS idx_memory_items_tags ON memory_items (tags)',
        'CREATE INDEX IF NOT EXISTS idx_memory_items_deleted ON memory_items (is_deleted)',
        
        'CREATE INDEX IF NOT EXISTS idx_episodic_timestamp ON episodic_memories (timestamp DESC)',
        'CREATE INDEX IF NOT EXISTS idx_episodic_importance ON episodic_memories (importance DESC)',
        'CREATE INDEX IF NOT EXISTS idx_episodic_participants ON episodic_memories (participants)',
        'CREATE INDEX IF NOT EXISTS idx_episodic_deleted ON episodic_memories (is_deleted)',
        
        'CREATE INDEX IF NOT EXISTS idx_semantic_concept ON semantic_memories (concept)',
        'CREATE INDEX IF NOT EXISTS idx_semantic_confidence ON semantic_memories (confidence DESC)',
        'CREATE INDEX IF NOT EXISTS idx_semantic_deleted ON semantic_memories (is_deleted)',
        
        'CREATE INDEX IF NOT EXISTS idx_procedural_name ON procedural_memories (name)',
        'CREATE INDEX IF NOT EXISTS idx_procedural_success_rate ON procedural_memories (success_rate DESC)',
        'CREATE INDEX IF NOT EXISTS idx_procedural_usage_count ON procedural_memories (usage_count DESC)',
        'CREATE INDEX IF NOT EXISTS idx_procedural_deleted ON procedural_memories (is_deleted)',
        
        'CREATE INDEX IF NOT EXISTS idx_relationships_source ON memory_relationships (source_id)',
        'CREATE INDEX IF NOT EXISTS idx_relationships_target ON memory_relationships (target_id)',
        'CREATE INDEX IF NOT EXISTS idx_relationships_type ON memory_relationships (relationship_type)',
        'CREATE INDEX IF NOT EXISTS idx_relationships_strength ON memory_relationships (strength DESC)',
        
        'CREATE INDEX IF NOT EXISTS idx_snapshots_created_at ON memory_snapshots (created_at DESC)',
        'CREATE INDEX IF NOT EXISTS idx_statistics_type_name ON memory_statistics (stat_type, stat_name)',
        'CREATE INDEX IF NOT EXISTS idx_statistics_timestamp ON memory_statistics (timestamp DESC)',
        'CREATE INDEX IF NOT EXISTS idx_operations_log_timestamp ON memory_operations_log (timestamp DESC)',
        'CREATE INDEX IF NOT EXISTS idx_operations_log_operation_type ON memory_operations_log (operation_type)'
    ]


class DatabaseManager:
    """数据库管理器"""
    
    def __init__(self, config: PersistenceConfig):
        self.config = config
        self.db_path = Path(config.database_path)
        self.connection_pool = []
        self.max_connections = config.max_connections
        self.active_connections = 0
        self.lock = asyncio.Lock()
        self.logger = logging.getLogger(f"{self.__class__.__name__}")
        
        # 确保数据库目录存在
        self.db_path.parent.mkdir(parents=True, exist_ok=True)
    
    async def initialize(self) -> None:
        """初始化数据库"""
        try:
            async with aiosqlite.connect(str(self.db_path)) as db:
                # 启用WAL模式
                if self.config.enable_wal:
                    await db.execute('PRAGMA journal_mode=WAL')
                
                # 启用外键约束
                if self.config.enable_foreign_keys:
                    await db.execute('PRAGMA foreign_keys=ON')
                
                # 设置其他优化参数
                await db.execute('PRAGMA synchronous=NORMAL')
                await db.execute('PRAGMA cache_size=10000')
                await db.execute('PRAGMA temp_store=MEMORY')
                
                # 创建表
                for table_name, table_sql in MemoryDatabaseSchema.TABLES.items():
                    await db.execute(table_sql)
                    self.logger.debug(f"Created/verified table: {table_name}")
                
                # 创建索引
                for index_sql in MemoryDatabaseSchema.INDEXES:
                    await db.execute(index_sql)
                
                await db.commit()
                self.logger.info("Database initialized successfully")
        
        except Exception as e:
            self.logger.error(f"Failed to initialize database: {e}")
            raise
    
    @asynccontextmanager
    async def get_connection(self):
        """获取数据库连接（连接池）"""
        async with self.lock:
            if self.connection_pool:
                connection = self.connection_pool.pop()
            else:
                if self.active_connections >= self.max_connections:
                    # 等待连接可用
                    while self.active_connections >= self.max_connections:
                        await asyncio.sleep(0.01)
                
                connection = await aiosqlite.connect(str(self.db_path))
                await connection.execute('PRAGMA foreign_keys=ON')
                self.active_connections += 1
        
        try:
            yield connection
        finally:
            async with self.lock:
                if len(self.connection_pool) < self.max_connections // 2:
                    self.connection_pool.append(connection)
                else:
                    await connection.close()
                    self.active_connections -= 1
    
    async def execute_query(self, query: str, params: Optional[Tuple] = None,
                          fetch_one: bool = False, fetch_all: bool = False) -> Any:
        """执行查询"""
        start_time = datetime.now()
        
        try:
            async with self.get_connection() as db:
                cursor = await db.execute(query, params or ())
                
                if fetch_one:
                    result = await cursor.fetchone()
                elif fetch_all:
                    result = await cursor.fetchall()
                else:
                    result = cursor.rowcount
                
                await db.commit()
                
                # 记录操作日志
                execution_time = (datetime.now() - start_time).total_seconds() * 1000
                await self._log_operation(
                    operation_type="QUERY",
                    table_name="unknown",
                    execution_time=execution_time,
                    success=True
                )
                
                return result
        
        except Exception as e:
            execution_time = (datetime.now() - start_time).total_seconds() * 1000
            await self._log_operation(
                operation_type="QUERY",
                table_name="unknown",
                execution_time=execution_time,
                success=False,
                error_message=str(e)
            )
            self.logger.error(f"Query execution failed: {e}")
            raise
    
    async def execute_batch(self, queries: List[Tuple[str, Tuple]]) -> List[Any]:
        """批量执行查询"""
        start_time = datetime.now()
        results = []
        
        try:
            async with self.get_connection() as db:
                for query, params in queries:
                    cursor = await db.execute(query, params)
                    results.append(cursor.rowcount)
                
                await db.commit()
                
                execution_time = (datetime.now() - start_time).total_seconds() * 1000
                await self._log_operation(
                    operation_type="BATCH",
                    table_name="multiple",
                    execution_time=execution_time,
                    success=True,
                    operation_data={"batch_size": len(queries)}
                )
                
                return results
        
        except Exception as e:
            execution_time = (datetime.now() - start_time).total_seconds() * 1000
            await self._log_operation(
                operation_type="BATCH",
                table_name="multiple",
                execution_time=execution_time,
                success=False,
                error_message=str(e),
                operation_data={"batch_size": len(queries)}
            )
            self.logger.error(f"Batch execution failed: {e}")
            raise
    
    async def _log_operation(self, operation_type: str, table_name: str,
                           execution_time: float, success: bool,
                           record_id: str = None, operation_data: Dict = None,
                           error_message: str = None) -> None:
        """记录操作日志"""
        try:
            log_id = str(uuid.uuid4())
            query = '''
                INSERT INTO memory_operations_log 
                (id, operation_type, table_name, record_id, operation_data, 
                 execution_time, success, error_message, timestamp)
                VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)
            '''
            params = (
                log_id, operation_type, table_name, record_id,
                json.dumps(operation_data) if operation_data else None,
                execution_time, success, error_message, datetime.now()
            )
            
            async with self.get_connection() as db:
                await db.execute(query, params)
                await db.commit()
        
        except Exception as e:
            # 记录日志失败不应该影响主要操作
            self.logger.warning(f"Failed to log operation: {e}")
    
    async def vacuum_database(self) -> None:
        """清理数据库"""
        try:
            async with self.get_connection() as db:
                await db.execute('VACUUM')
                self.logger.info("Database vacuum completed")
        except Exception as e:
            self.logger.error(f"Database vacuum failed: {e}")
    
    async def get_database_stats(self) -> Dict[str, Any]:
        """获取数据库统计信息"""
        stats = {}
        
        try:
            # 获取表大小信息
            for table_name in MemoryDatabaseSchema.TABLES.keys():
                count_query = f"SELECT COUNT(*) FROM {table_name}"
                count = await self.execute_query(count_query, fetch_one=True)
                stats[f"{table_name}_count"] = count[0] if count else 0
            
            # 获取数据库文件大小
            if self.db_path.exists():
                stats["database_size_bytes"] = self.db_path.stat().st_size
            
            # 获取最近的操作统计
            recent_ops_query = '''
                SELECT operation_type, COUNT(*), AVG(execution_time)
                FROM memory_operations_log 
                WHERE timestamp > datetime('now', '-1 hour')
                GROUP BY operation_type
            '''
            recent_ops = await self.execute_query(recent_ops_query, fetch_all=True)
            stats["recent_operations"] = {
                op_type: {"count": count, "avg_time": avg_time}
                for op_type, count, avg_time in recent_ops
            }
            
            return stats
        
        except Exception as e:
            self.logger.error(f"Failed to get database stats: {e}")
            return {}


class MemoryPersistenceManager:
    """记忆持久化管理器"""
    
    def __init__(self, config: Optional[PersistenceConfig] = None):
        self.config = config or PersistenceConfig()
        self.db_manager = DatabaseManager(self.config)
        self.pending_operations = []
        self.last_batch_time = datetime.now()
        self.batch_task = None
        self.is_running = False
        
        # 压缩和加密支持
        self.compression_enabled = self.config.enable_compression
        self.encryption_enabled = self.config.enable_encryption
        
        self.logger = logging.getLogger(f"{self.__class__.__name__}")
    
    async def initialize(self) -> None:
        """初始化持久化管理器"""
        await self.db_manager.initialize()
        
        if self.config.persistence_mode == PersistenceMode.BATCH:
            self.is_running = True
            self.batch_task = asyncio.create_task(self._batch_processing_loop())
        
        self.logger.info("Memory persistence manager initialized")
    
    async def shutdown(self) -> None:
        """关闭持久化管理器"""
        self.is_running = False
        
        if self.batch_task:
            self.batch_task.cancel()
            try:
                await self.batch_task
            except asyncio.CancelledError:
                pass
        
        # 处理剩余的待处理操作
        if self.pending_operations:
            await self._flush_batch()
        
        self.logger.info("Memory persistence manager shut down")
    
    async def store_memory_item(self, memory_item: MemoryItem) -> str:
        """存储记忆项"""
        operation = {
            "type": "store_memory_item",
            "data": memory_item,
            "timestamp": datetime.now()
        }
        
        if self.config.persistence_mode == PersistenceMode.IMMEDIATE:
            return await self._execute_store_memory_item(memory_item)
        else:
            self.pending_operations.append(operation)
            return memory_item.item_id
    
    async def _execute_store_memory_item(self, memory_item: MemoryItem) -> str:
        """执行存储记忆项"""
        try:
            # 序列化复杂数据
            tags_json = json.dumps(memory_item.tags) if memory_item.tags else None
            metadata_json = json.dumps(memory_item.metadata) if memory_item.metadata else None
            embedding_blob = pickle.dumps(memory_item.embedding) if memory_item.embedding is not None else None
            keywords_json = json.dumps(memory_item.semantic_keywords) if memory_item.semantic_keywords else None
            
            # 计算内容哈希
            content_str = str(memory_item.content)
            content_hash = hashlib.sha256(content_str.encode()).hexdigest()
            
            query = '''
                INSERT OR REPLACE INTO memory_items 
                (id, content, content_hash, memory_type, importance, access_count, 
                 last_accessed, created_at, updated_at, tags, metadata, embedding, 
                 embedding_model, semantic_keywords, is_deleted, version)
                VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
            '''
            
            params = (
                memory_item.item_id,
                content_str,
                content_hash,
                memory_item.memory_type.value,
                memory_item.importance,
                memory_item.access_count,
                memory_item.last_accessed,
                memory_item.created_at,
                datetime.now(),  # updated_at
                tags_json,
                metadata_json,
                embedding_blob,
                memory_item.embedding_model,
                keywords_json,
                False,  # is_deleted
                1       # version
            )
            
            await self.db_manager.execute_query(query, params)
            
            self.logger.debug(f"Stored memory item: {memory_item.item_id}")
            return memory_item.item_id
        
        except Exception as e:
            self.logger.error(f"Failed to store memory item {memory_item.item_id}: {e}")
            raise
    
    async def retrieve_memory_item(self, item_id: str) -> Optional[MemoryItem]:
        """检索记忆项"""
        try:
            query = '''
                SELECT id, content, memory_type, importance, access_count, 
                       last_accessed, created_at, tags, metadata, embedding,
                       embedding_model, semantic_keywords
                FROM memory_items 
                WHERE id = ? AND is_deleted = FALSE
            '''
            
            row = await self.db_manager.execute_query(query, (item_id,), fetch_one=True)
            
            if not row:
                return None
            
            # 反序列化数据
            tags = json.loads(row[7]) if row[7] else []
            metadata = json.loads(row[8]) if row[8] else {}
            embedding = pickle.loads(row[9]) if row[9] else None
            semantic_keywords = json.loads(row[11]) if row[11] else []
            
            # 更新访问计数
            await self._update_access_count(item_id)
            
            memory_item = MemoryItem(
                item_id=row[0],
                content=row[1],
                memory_type=MemoryType(row[2]),
                importance=row[3],
                access_count=row[4] + 1,  # 增加访问计数
                last_accessed=datetime.now(),
                created_at=row[6],
                tags=tags,
                metadata=metadata,
                embedding=embedding,
                embedding_model=row[10],
                semantic_keywords=semantic_keywords
            )
            
            return memory_item
        
        except Exception as e:
            self.logger.error(f"Failed to retrieve memory item {item_id}: {e}")
            return None
    
    async def _update_access_count(self, item_id: str) -> None:
        """更新访问计数"""
        try:
            query = '''
                UPDATE memory_items 
                SET access_count = access_count + 1, last_accessed = ?
                WHERE id = ?
            '''
            await self.db_manager.execute_query(query, (datetime.now(), item_id))
        except Exception as e:
            self.logger.warning(f"Failed to update access count for {item_id}: {e}")
    
    async def search_memory_items(self, 
                                 memory_types: Optional[List[MemoryType]] = None,
                                 tags: Optional[List[str]] = None,
                                 min_importance: float = 0.0,
                                 max_results: int = 100,
                                 order_by: str = "importance DESC") -> List[MemoryItem]:
        """搜索记忆项"""
        try:
            # 构建查询条件
            conditions = ["is_deleted = FALSE"]
            params = []
            
            if memory_types:
                type_placeholders = ",".join("?" * len(memory_types))
                conditions.append(f"memory_type IN ({type_placeholders})")
                params.extend([mt.value for mt in memory_types])
            
            if tags:
                for tag in tags:
                    conditions.append("tags LIKE ?")
                    params.append(f'%"{tag}"%')
            
            if min_importance > 0:
                conditions.append("importance >= ?")
                params.append(min_importance)
            
            where_clause = " AND ".join(conditions)
            
            query = f'''
                SELECT id, content, memory_type, importance, access_count, 
                       last_accessed, created_at, tags, metadata, embedding,
                       embedding_model, semantic_keywords
                FROM memory_items 
                WHERE {where_clause}
                ORDER BY {order_by}
                LIMIT ?
            '''
            params.append(max_results)
            
            rows = await self.db_manager.execute_query(query, tuple(params), fetch_all=True)
            
            memory_items = []
            for row in rows:
                tags = json.loads(row[7]) if row[7] else []
                metadata = json.loads(row[8]) if row[8] else {}
                embedding = pickle.loads(row[9]) if row[9] else None
                semantic_keywords = json.loads(row[11]) if row[11] else []
                
                memory_item = MemoryItem(
                    item_id=row[0],
                    content=row[1],
                    memory_type=MemoryType(row[2]),
                    importance=row[3],
                    access_count=row[4],
                    last_accessed=row[5],
                    created_at=row[6],
                    tags=tags,
                    metadata=metadata,
                    embedding=embedding,
                    embedding_model=row[10],
                    semantic_keywords=semantic_keywords
                )
                memory_items.append(memory_item)
            
            return memory_items
        
        except Exception as e:
            self.logger.error(f"Failed to search memory items: {e}")
            return []
    
    async def store_episodic_memory(self, episodic_memory: EpisodicMemory) -> str:
        """存储情景记忆"""
        try:
            query = '''
                INSERT OR REPLACE INTO episodic_memories 
                (id, episode_id, event, context, participants, location, timestamp,
                 emotional_valence, importance, related_episodes, metadata, 
                 created_at, updated_at, is_deleted)
                VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
            '''
            
            record_id = str(uuid.uuid4())
            params = (
                record_id,
                episodic_memory.episode_id,
                episodic_memory.event,
                json.dumps(episodic_memory.context),
                json.dumps(episodic_memory.participants),
                episodic_memory.location,
                episodic_memory.timestamp,
                episodic_memory.emotional_valence,
                episodic_memory.importance,
                json.dumps(episodic_memory.related_episodes),
                json.dumps(episodic_memory.metadata),
                datetime.now(),
                datetime.now(),
                False
            )
            
            await self.db_manager.execute_query(query, params)
            return record_id
        
        except Exception as e:
            self.logger.error(f"Failed to store episodic memory: {e}")
            raise
    
    async def store_semantic_memory(self, semantic_memory: SemanticMemory) -> str:
        """存储语义记忆"""
        try:
            query = '''
                INSERT OR REPLACE INTO semantic_memories 
                (id, concept_id, concept, definition, properties, relationships,
                 confidence, source, created_at, updated_at, metadata, is_deleted)
                VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
            '''
            
            record_id = str(uuid.uuid4())
            params = (
                record_id,
                semantic_memory.concept_id,
                semantic_memory.concept,
                semantic_memory.definition,
                json.dumps(semantic_memory.properties),
                json.dumps(semantic_memory.relationships),
                semantic_memory.confidence,
                semantic_memory.source,
                semantic_memory.created_at,
                datetime.now(),
                json.dumps(semantic_memory.metadata),
                False
            )
            
            await self.db_manager.execute_query(query, params)
            return record_id
        
        except Exception as e:
            self.logger.error(f"Failed to store semantic memory: {e}")
            raise
    
    async def store_procedural_memory(self, procedural_memory: ProceduralMemory) -> str:
        """存储程序记忆"""
        try:
            query = '''
                INSERT OR REPLACE INTO procedural_memories 
                (id, procedure_id, name, description, steps, preconditions,
                 postconditions, success_rate, usage_count, last_used,
                 created_at, updated_at, metadata, is_deleted)
                VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
            '''
            
            record_id = str(uuid.uuid4())
            params = (
                record_id,
                procedural_memory.procedure_id,
                procedural_memory.name,
                procedural_memory.description,
                json.dumps(procedural_memory.steps),
                json.dumps(procedural_memory.preconditions),
                json.dumps(procedural_memory.postconditions),
                procedural_memory.success_rate,
                procedural_memory.usage_count,
                procedural_memory.last_used,
                procedural_memory.created_at,
                datetime.now(),
                json.dumps(procedural_memory.metadata),
                False
            )
            
            await self.db_manager.execute_query(query, params)
            return record_id
        
        except Exception as e:
            self.logger.error(f"Failed to store procedural memory: {e}")
            raise
    
    async def create_memory_snapshot(self, name: str, data: Dict[str, Any]) -> str:
        """创建记忆快照"""
        try:
            snapshot_id = str(uuid.uuid4())
            
            # 序列化和可选压缩
            data_json = json.dumps(data, default=str)
            compressed_data = data_json
            compression_type = "none"
            
            if self.compression_enabled:
                import gzip
                compressed_data = gzip.compress(data_json.encode()).decode('latin-1')
                compression_type = "gzip"
            
            # 计算校验和
            checksum = hashlib.md5(compressed_data.encode()).hexdigest()
            
            query = '''
                INSERT INTO memory_snapshots 
                (id, name, snapshot_data, compression_type, size_bytes, checksum, created_at, metadata)
                VALUES (?, ?, ?, ?, ?, ?, ?, ?)
            '''
            
            params = (
                snapshot_id,
                name,
                compressed_data,
                compression_type,
                len(compressed_data),
                checksum,
                datetime.now(),
                json.dumps({"original_size": len(data_json)})
            )
            
            await self.db_manager.execute_query(query, params)
            
            self.logger.info(f"Created memory snapshot: {snapshot_id}")
            return snapshot_id
        
        except Exception as e:
            self.logger.error(f"Failed to create memory snapshot: {e}")
            raise
    
    async def restore_memory_snapshot(self, snapshot_id: str) -> Optional[Dict[str, Any]]:
        """恢复记忆快照"""
        try:
            query = '''
                SELECT snapshot_data, compression_type, checksum
                FROM memory_snapshots 
                WHERE id = ?
            '''
            
            row = await self.db_manager.execute_query(query, (snapshot_id,), fetch_one=True)
            
            if not row:
                return None
            
            snapshot_data, compression_type, stored_checksum = row
            
            # 验证校验和
            actual_checksum = hashlib.md5(snapshot_data.encode()).hexdigest()
            if actual_checksum != stored_checksum:
                raise ValueError(f"Snapshot {snapshot_id} checksum mismatch")
            
            # 解压缩
            if compression_type == "gzip":
                import gzip
                decompressed_data = gzip.decompress(snapshot_data.encode('latin-1')).decode()
            else:
                decompressed_data = snapshot_data
            
            return json.loads(decompressed_data)
        
        except Exception as e:
            self.logger.error(f"Failed to restore memory snapshot {snapshot_id}: {e}")
            return None
    
    async def _batch_processing_loop(self) -> None:
        """批量处理循环"""
        while self.is_running:
            try:
                await asyncio.sleep(1.0)  # 检查间隔
                
                current_time = datetime.now()
                time_since_last_batch = (current_time - self.last_batch_time).total_seconds()
                
                should_process = (
                    len(self.pending_operations) >= self.config.batch_size or
                    time_since_last_batch >= self.config.batch_timeout
                )
                
                if should_process and self.pending_operations:
                    await self._flush_batch()
            
            except asyncio.CancelledError:
                break
            except Exception as e:
                self.logger.error(f"Error in batch processing loop: {e}")
    
    async def _flush_batch(self) -> None:
        """刷新批处理队列"""
        if not self.pending_operations:
            return
        
        try:
            batch_queries = []
            
            for operation in self.pending_operations:
                if operation["type"] == "store_memory_item":
                    memory_item = operation["data"]
                    
                    # 构建查询和参数
                    tags_json = json.dumps(memory_item.tags) if memory_item.tags else None
                    metadata_json = json.dumps(memory_item.metadata) if memory_item.metadata else None
                    embedding_blob = pickle.dumps(memory_item.embedding) if memory_item.embedding is not None else None
                    keywords_json = json.dumps(memory_item.semantic_keywords) if memory_item.semantic_keywords else None
                    content_str = str(memory_item.content)
                    content_hash = hashlib.sha256(content_str.encode()).hexdigest()
                    
                    query = '''
                        INSERT OR REPLACE INTO memory_items 
                        (id, content, content_hash, memory_type, importance, access_count, 
                         last_accessed, created_at, updated_at, tags, metadata, embedding, 
                         embedding_model, semantic_keywords, is_deleted, version)
                        VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
                    '''
                    
                    params = (
                        memory_item.item_id, content_str, content_hash,
                        memory_item.memory_type.value, memory_item.importance,
                        memory_item.access_count, memory_item.last_accessed,
                        memory_item.created_at, datetime.now(), tags_json,
                        metadata_json, embedding_blob, memory_item.embedding_model,
                        keywords_json, False, 1
                    )
                    
                    batch_queries.append((query, params))
            
            if batch_queries:
                await self.db_manager.execute_batch(batch_queries)
                self.logger.info(f"Flushed batch of {len(batch_queries)} operations")
            
            self.pending_operations.clear()
            self.last_batch_time = datetime.now()
        
        except Exception as e:
            self.logger.error(f"Failed to flush batch: {e}")
    
    async def cleanup_old_data(self, retention_days: int = None) -> Dict[str, int]:
        """清理旧数据"""
        retention_days = retention_days or self.config.retention_days
        cutoff_date = datetime.now() - timedelta(days=retention_days)
        
        cleanup_results = {}
        
        try:
            # 清理旧的操作日志
            log_query = "DELETE FROM memory_operations_log WHERE timestamp < ?"
            log_count = await self.db_manager.execute_query(log_query, (cutoff_date,))
            cleanup_results["operations_log_deleted"] = log_count
            
            # 清理旧的统计数据
            stats_query = "DELETE FROM memory_statistics WHERE timestamp < ?"
            stats_count = await self.db_manager.execute_query(stats_query, (cutoff_date,))
            cleanup_results["statistics_deleted"] = stats_count
            
            # 软删除旧的低重要性记忆项
            memory_query = '''
                UPDATE memory_items 
                SET is_deleted = TRUE 
                WHERE created_at < ? AND importance < 0.3 AND access_count < 2
            '''
            memory_count = await self.db_manager.execute_query(memory_query, (cutoff_date,))
            cleanup_results["memory_items_archived"] = memory_count
            
            self.logger.info(f"Cleanup completed: {cleanup_results}")
            return cleanup_results
        
        except Exception as e:
            self.logger.error(f"Failed to cleanup old data: {e}")
            return {}
    
    async def get_persistence_statistics(self) -> Dict[str, Any]:
        """获取持久化统计信息"""
        try:
            db_stats = await self.db_manager.get_database_stats()
            
            return {
                "database_stats": db_stats,
                "pending_operations": len(self.pending_operations),
                "persistence_mode": self.config.persistence_mode.value,
                "batch_size": self.config.batch_size,
                "last_batch_time": self.last_batch_time.isoformat(),
                "config": asdict(self.config)
            }
        
        except Exception as e:
            self.logger.error(f"Failed to get persistence statistics: {e}")
            return {} 


"""
A2A Protocol Integration Tests
测试A2A协议与适配器的集成功能
"""

import pytest
import asyncio
from datetime import datetime
from unittest.mock import Mock, AsyncMock, patch
import pytest_asyncio

from layers.framework.abstractions.a2a_protocol import (
    A2AMessageType,
    A2ACapabilityType,
    A2AProtocolVersion,
    A2ACapability,
    A2AAgentProfile,
    A2AMessage,
    A2AHTTPTransport,
    A2AProtocolHandler,
    create_a2a_capability,
    create_a2a_agent_profile
)
from layers.framework.abstractions.a2a_integration import (
    A2AIntegrationManager,
    A2AAdapterBridge,
    A2AMessageRouter
)
from layers.framework.abstractions.task import TaskType, UniversalTask
from layers.framework.abstractions.result import ResultStatus, UniversalResult
from layers.framework.abstractions.context import UniversalContext


class MockAdapter:
    """模拟适配器"""
    def __init__(self, name="mock_adapter"):
        self.name = name
        self.info = {
            "name": name,
            "version": "1.0",
            "capabilities": ["text_processing", "code_generation", "task_coordination"]
        }
    
    def get_info(self):
        return Mock(**self.info)
    
    async def execute_task(self, task, context):
        return UniversalResult(
            content={"reply": "Mock response"},
            status=ResultStatus.SUCCESS,
            result_type=TaskType.CONVERSATION
        )


@pytest.mark.asyncio
class TestA2AIntegration:
    """测试A2A协议与适配器的集成"""
    
    @pytest_asyncio.fixture
    async def integration_setup(self):
        """设置集成测试环境"""
        # 创建模拟适配器
        adapter = MockAdapter()
        
        # 创建A2A协议处理器
        capability = create_a2a_capability(A2ACapabilityType.TEXT_PROCESSING)
        profile = create_a2a_agent_profile(
            agent_id="test_agent",
            agent_name="Test Agent",
            capabilities=[capability]
        )
        transport = A2AHTTPTransport()
        protocol_handler = A2AProtocolHandler(profile, transport)
        
        # 创建集成管理器
        integration_manager = A2AIntegrationManager()
        
        # 创建适配器桥接器
        adapter_bridge = A2AAdapterBridge(adapter, protocol_handler)
        
        # 创建消息路由器
        message_router = A2AMessageRouter()
        
        # 注册组件
        integration_manager.register_adapter_bridge(adapter_bridge)
        integration_manager.register_message_router(message_router)
        
        # 注册测试用的授权Agent
        integration_manager._authorized_agents.add("agent1")
        integration_manager._authorized_agents.add("test_agent")
        
        await transport.connect("http://test-endpoint")
        yield {
            "adapter": adapter,
            "protocol_handler": protocol_handler,
            "integration_manager": integration_manager,
            "adapter_bridge": adapter_bridge,
            "message_router": message_router
        }
        await transport.disconnect()
    
    async def test_adapter_bridge_initialization(self, integration_setup):
        """测试适配器桥接器初始化"""
        bridge = integration_setup["adapter_bridge"]
        
        assert bridge.adapter is not None
        assert bridge.protocol_handler is not None
        assert bridge.is_ready()
        
        # 验证能力映射
        capabilities = bridge.get_mapped_capabilities()
        assert len(capabilities) > 0
        assert any(cap.capability_type == A2ACapabilityType.TEXT_PROCESSING for cap in capabilities)
    
    async def test_message_routing(self, integration_setup):
        """测试消息路由"""
        router = integration_setup["message_router"]
        protocol_handler = integration_setup["protocol_handler"]
        
        # 创建测试消息
        message = A2AMessage(
            message_id="test_route",
            protocol_version=A2AProtocolVersion.V2_0,
            message_type=A2AMessageType.TASK_REQUEST,
            sender_id="agent1",
            receiver_id="agent2",
            timestamp=datetime.now(),
            payload={"task": "test routing"}
        )
        
        # 注册路由处理器
        async def test_handler(msg):
            return A2AMessage(
                message_id="response",
                protocol_version=A2AProtocolVersion.V2_0,
                message_type=A2AMessageType.TASK_RESPONSE,
                sender_id="agent2",
                receiver_id="agent1",
                timestamp=datetime.now(),
                correlation_id=msg.message_id,
                payload={"result": "routed"}
            )
        
        router.register_handler(A2AMessageType.TASK_REQUEST, test_handler)
        
        # 测试路由
        response = await router.route_message(message)
        assert response is not None
        assert response.message_type == A2AMessageType.TASK_RESPONSE
        assert response.correlation_id == "test_route"
    
    async def test_task_execution_through_bridge(self, integration_setup):
        """测试通过桥接器执行任务"""
        bridge = integration_setup["adapter_bridge"]
        
        # 创建任务请求消息
        task_message = A2AMessage(
            message_id="test_task",
            protocol_version=A2AProtocolVersion.V2_0,
            message_type=A2AMessageType.TASK_REQUEST,
            sender_id="agent1",
            receiver_id="test_agent",
            timestamp=datetime.now(),
            payload={
                "task_type": "conversation",
                "content": "Hello, how are you?",
                "parameters": {}
            }
        )
        
        # 执行任务
        result = await bridge.execute_task(task_message)
        
        assert result is not None
        assert result.message_type == A2AMessageType.TASK_RESPONSE
        assert result.correlation_id == "test_task"
        assert "reply" in result.payload
    
    async def test_capability_mapping(self, integration_setup):
        """测试能力映射"""
        bridge = integration_setup["adapter_bridge"]
        
        # 获取适配器能力
        adapter_capabilities = bridge.adapter.get_info().capabilities
        
        # 获取映射后的A2A能力
        a2a_capabilities = bridge.get_mapped_capabilities()
        
        # 验证映射关系
        assert len(adapter_capabilities) > 0
        assert len(a2a_capabilities) > 0
        
        # 验证核心能力是否正确映射
        core_capabilities = {
            "text_processing": A2ACapabilityType.TEXT_PROCESSING,
            "code_generation": A2ACapabilityType.CODE_GENERATION,
            "task_coordination": A2ACapabilityType.TASK_COORDINATION
        }
        
        for adapter_cap in adapter_capabilities:
            if adapter_cap in core_capabilities:
                assert any(cap.capability_type == core_capabilities[adapter_cap] 
                         for cap in a2a_capabilities)
    
    async def test_integration_manager_coordination(self, integration_setup):
        """测试集成管理器的协调功能"""
        manager = integration_setup["integration_manager"]
        
        # 创建测试消息
        message = A2AMessage(
            message_id="test_coordination",
            protocol_version=A2AProtocolVersion.V2_0,
            message_type=A2AMessageType.COLLABORATION_INVITE,
            sender_id="agent1",
            receiver_id="test_agent",
            timestamp=datetime.now(),
            payload={
                "collaboration_type": "team_task",
                "task": "Solve this problem together",
                "participants": ["agent1", "test_agent"],
                "collaboration_id": "test_coordination"  # 显式设置ID
            }
        )
        
        # 测试消息处理和协调
        result = await manager.handle_message(message)
        
        assert result is not None
        assert result.message_type == A2AMessageType.COLLABORATION_ACCEPT
        assert result.correlation_id == "test_coordination"
        
        # 验证协作状态
        collaboration = manager.get_active_collaboration("test_coordination")
        assert collaboration is not None
        assert collaboration["status"] == "active"
        assert len(collaboration["participants"]) == 2
    
    async def test_error_handling_and_recovery(self, integration_setup):
        """测试错误处理和恢复机制"""
        bridge = integration_setup["adapter_bridge"]
        manager = integration_setup["integration_manager"]
        
        # 模拟错误情况
        with patch.object(bridge.adapter, 'execute_task', 
                         side_effect=Exception("Simulated error")):
            
            # 创建任务消息
            message = A2AMessage(
                message_id="test_error",
                protocol_version=A2AProtocolVersion.V2_0,
                message_type=A2AMessageType.TASK_REQUEST,
                sender_id="agent1",
                receiver_id="test_agent",
                timestamp=datetime.now(),
                payload={"task": "This will fail"}
            )
            
            # 测试错误处理
            result = await manager.handle_message(message)
            
            assert result is not None
            assert result.message_type == A2AMessageType.ERROR_REPORT
            assert result.payload["error_type"] == "execution_error"
            assert "Simulated error" in result.payload["error_message"]
            
            # 验证错误恢复
            assert manager.is_healthy()
            assert bridge.is_ready()
    
    async def test_message_validation_and_security(self, integration_setup):
        """测试消息验证和安全机制"""
        manager = integration_setup["integration_manager"]
        
        # 测试无效消息
        invalid_message = A2AMessage(
            message_id="test_invalid",
            protocol_version=A2AProtocolVersion.V2_0,
            message_type=A2AMessageType.TASK_REQUEST,
            sender_id="unknown_agent",  # 未注册的发送者
            receiver_id="test_agent",
            timestamp=datetime.now(),
            payload={"task": "Invalid request"}
        )
        
        # 验证消息处理
        result = await manager.handle_message(invalid_message)
        
        assert result is not None
        assert result.message_type == A2AMessageType.ERROR_REPORT
        assert result.payload["error_type"] == "unauthorized"
        
        # 测试消息完整性
        tampered_message = A2AMessage(
            message_id="test_tampered",
            protocol_version=A2AProtocolVersion.V2_0,
            message_type=A2AMessageType.TASK_REQUEST,
            sender_id="agent1",  # 使用授权的发送者
            receiver_id="test_agent",
            timestamp=datetime.now(),
            payload={"task": "Tampered request"},
            signature="invalid_signature"  # 无效签名
        )
        
        result = await manager.handle_message(tampered_message)
        
        assert result is not None
        assert result.message_type == A2AMessageType.ERROR_REPORT
        assert result.payload["error_type"] == "integrity_error" 


"""
Unit tests for CLI application layer
"""

import pytest
import argparse
import asyncio
import sys
import io
from unittest.mock import Mock, patch, MagicMock
from pathlib import Path

# Add project root to path for imports
project_root = Path(__file__).parent.parent.parent.parent
sys.path.insert(0, str(project_root))

from layers.application.cli.main import ADCCLIApp
from layers.application.cli.commands import CommandRegistry
from layers.application.cli.interactive import InteractiveShell


class TestADCCLIApp:
    """Test ADC CLI Application"""
    
    def test_initialization(self):
        """Test CLI app initialization"""
        app = ADCCLIApp()
        
        assert hasattr(app, 'config_manager')
        assert hasattr(app, 'logger')
        assert hasattr(app, 'command_registry')
        assert hasattr(app, 'interactive_shell')
        assert app.interactive_shell is None
    
    def test_create_parser(self):
        """Test argument parser creation"""
        app = ADCCLIApp()
        parser = app.create_parser()
        
        assert isinstance(parser, argparse.ArgumentParser)
        assert parser.prog == "adc"
        assert "Zeus" in parser.description
    
    def test_parser_version_argument(self):
        """Test version argument parsing"""
        app = ADCCLIApp()
        parser = app.create_parser()
        
        # Test version argument
        with pytest.raises(SystemExit) as exc_info:
            with patch('sys.stdout', new_callable=io.StringIO) as mock_stdout:
                parser.parse_args(['--version'])
        
        # SystemExit with code 0 indicates successful version display
        assert exc_info.value.code == 0
    
    def test_parser_interactive_argument(self):
        """Test interactive argument parsing"""
        app = ADCCLIApp()
        parser = app.create_parser()
        
        args = parser.parse_args(['--interactive'])
        assert args.interactive is True
    
    def test_parser_help_argument(self):
        """Test help argument parsing"""
        app = ADCCLIApp()
        parser = app.create_parser()
        
        with pytest.raises(SystemExit) as exc_info:
            with patch('sys.stdout', new_callable=io.StringIO):
                parser.parse_args(['--help'])
        
        assert exc_info.value.code == 0
    
    @pytest.mark.asyncio
    async def test_run_async_with_command(self):
        """Test async run with command"""
        app = ADCCLIApp()
        
        # Mock successful command execution
        with patch.object(app.command_registry, 'execute_command', return_value=0) as mock_execute:
            args = Mock()
            args.log_level = 'INFO'
            args.verbose = False
            args.config = None
            args.interactive = False
            args.command = 'demo'
            
            result = await app.run_async(args)
            
            assert result == 0
            mock_execute.assert_called_once()
    
    @pytest.mark.asyncio
    async def test_run_async_with_interactive(self):
        """Test async run with interactive mode"""
        app = ADCCLIApp()
        
        # Mock interactive shell
        mock_shell = Mock()
        mock_shell.run = Mock(return_value=0)
        
        with patch('layers.application.cli.interactive.InteractiveShell', return_value=mock_shell):
            args = Mock()
            args.log_level = 'INFO'
            args.verbose = False
            args.config = None
            args.interactive = True
            args.command = None
            
            result = await app.run_async(args)
            
            assert result == 0
            assert app.interactive_shell is not None
            mock_shell.run.assert_called_once()
    
    @pytest.mark.asyncio
    async def test_run_async_no_command(self):
        """Test async run without command (shows help)"""
        app = ADCCLIApp()
        
        with patch.object(app, 'create_parser') as mock_parser_creator:
            mock_parser = Mock()
            mock_parser.print_help = Mock()
            mock_parser_creator.return_value = mock_parser
            
            args = Mock()
            args.log_level = 'INFO'
            args.verbose = False
            args.config = None
            args.interactive = False
            args.command = None
            
            result = await app.run_async(args)
            
            assert result == 0
            mock_parser.print_help.assert_called_once()


class TestCommandRegistry:
    """Test Command Registry"""
    
    def test_initialization(self):
        """Test command registry initialization"""
        registry = CommandRegistry()
        
        assert hasattr(registry, 'commands')
        assert isinstance(registry.commands, dict)
    
    def test_register_commands(self):
        """Test command registration"""
        registry = CommandRegistry()
        
        # Create a mock argument parser with subparsers
        parser = argparse.ArgumentParser()
        subparsers = parser.add_subparsers()
        
        # Should not raise any exceptions
        registry.register_commands(subparsers)
        
        # Verify that subcommands were added
        assert len(subparsers.choices) > 0
        assert 'agent' in subparsers.choices
        assert 'workflow' in subparsers.choices
        assert 'team' in subparsers.choices
        assert 'demo' in subparsers.choices
        assert 'system' in subparsers.choices
    
    @pytest.mark.asyncio
    async def test_execute_agent_list_command(self):
        """Test agent list command execution"""
        registry = CommandRegistry()
        
        # Mock arguments for agent list command
        args = Mock()
        args.command = 'agent'
        args.agent_action = 'list'
        args.format = 'table'
        
        with patch('builtins.print') as mock_print:
            result = await registry.execute_command(args)
            
            # Should execute successfully (return 0)
            assert result == 0
            # Should print some output
            mock_print.assert_called()
    
    @pytest.mark.asyncio
    async def test_execute_workflow_list_command(self):
        """Test workflow list command execution"""
        registry = CommandRegistry()
        
        # Mock arguments for workflow list command
        args = Mock()
        args.command = 'workflow'
        args.workflow_action = 'list'
        args.format = 'table'
        
        with patch('builtins.print') as mock_print:
            result = await registry.execute_command(args)
            
            assert result == 0
            mock_print.assert_called()
    
    @pytest.mark.asyncio
    async def test_execute_team_list_command(self):
        """Test team list command execution"""
        registry = CommandRegistry()
        
        # Mock arguments for team list command
        args = Mock()
        args.command = 'team'
        args.team_action = 'list'
        args.format = 'table'
        
        with patch('builtins.print') as mock_print:
            result = await registry.execute_command(args)
            
            assert result == 0
            mock_print.assert_called()
    
    @pytest.mark.asyncio
    async def test_execute_system_info_command(self):
        """Test system info command execution"""
        registry = CommandRegistry()
        
        # Mock arguments for system info command
        args = Mock()
        args.command = 'system'
        args.system_action = 'info'
        
        with patch('builtins.print') as mock_print:
            result = await registry.execute_command(args)
            
            assert result == 0
            mock_print.assert_called()
    
    @pytest.mark.asyncio
    async def test_execute_system_health_command(self):
        """Test system health command execution"""
        registry = CommandRegistry()
        
        # Mock arguments for system health command
        args = Mock()
        args.command = 'system'
        args.system_action = 'health'
        
        with patch('builtins.print') as mock_print:
            result = await registry.execute_command(args)
            
            assert result == 0
            mock_print.assert_called()
    
    @pytest.mark.asyncio
    async def test_execute_demo_basic_command(self):
        """Test demo basic command execution"""
        registry = CommandRegistry()
        
        # Mock arguments for demo command
        args = Mock()
        args.command = 'demo'
        args.demo_type = 'basic'
        
        with patch('builtins.print') as mock_print:
            result = await registry.execute_command(args)
            
            assert result == 0
            mock_print.assert_called()
    
    @pytest.mark.asyncio
    async def test_execute_invalid_command(self):
        """Test execution of invalid command"""
        registry = CommandRegistry()
        
        # Mock arguments for invalid command
        args = Mock()
        args.command = 'invalid'
        
        with patch('builtins.print') as mock_print:
            result = await registry.execute_command(args)
            
            # Should return 1 for invalid command
            assert result == 1
            mock_print.assert_called()
    
    @pytest.mark.asyncio
    async def test_execute_command_with_exception(self):
        """Test command execution with exception"""
        registry = CommandRegistry()
        
        # Mock arguments that will cause an exception
        args = Mock()
        args.command = 'agent'
        args.agent_action = 'create'
        args.name = 'test_agent'
        args.type = 'openai'
        args.model = 'gpt-4o-mini'
        args.system_message = None
        
        # Mock the OpenAI adapter to raise an exception
        with patch('layers.application.cli.commands.OpenAIAdapter') as mock_adapter:
            mock_adapter.side_effect = Exception("Connection error")
            
            with patch('builtins.print') as mock_print:
                result = await registry.execute_command(args)
                
                # Should handle exception gracefully and return 1
                assert result == 1
                mock_print.assert_called()


class TestInteractiveShell:
    """Test Interactive Shell"""
    
    def test_initialization(self):
        """Test interactive shell initialization"""
        registry = CommandRegistry()
        shell = InteractiveShell(registry)
        
        assert hasattr(shell, 'command_registry')
        assert shell.command_registry == registry
        assert hasattr(shell, 'history_file')
        assert hasattr(shell, 'running')
        assert hasattr(shell, 'builtin_commands')
    
    def test_convert_to_argv(self):
        """Test command conversion to argv format"""
        registry = CommandRegistry()
        shell = InteractiveShell(registry)
        
        result = shell._convert_to_argv(["agent", "list"])
        
        assert isinstance(result, list)
        assert result == ["agent", "list"]
    
    def test_get_prompt(self):
        """Test prompt generation"""
        registry = CommandRegistry()
        shell = InteractiveShell(registry)
        
        prompt = shell._get_prompt()
        
        assert isinstance(prompt, str)
        assert "adc>" in prompt
    
    @pytest.mark.asyncio
    async def test_builtin_help_command(self):
        """Test built-in help command"""
        registry = CommandRegistry()
        shell = InteractiveShell(registry)
        
        with patch('builtins.print') as mock_print:
            await shell._cmd_help([])
            
            mock_print.assert_called()
            # Should print help information
            calls = [call.args[0] for call in mock_print.call_args_list]
            help_text = ' '.join(str(call) for call in calls)
            assert "可用命令" in help_text or "Agent管理" in help_text
    
    @pytest.mark.asyncio
    async def test_builtin_version_command(self):
        """Test built-in version command"""
        registry = CommandRegistry()
        shell = InteractiveShell(registry)
        
        with patch('builtins.print') as mock_print:
            await shell._cmd_version([])
            
            mock_print.assert_called()
    
    @pytest.mark.asyncio
    async def test_execute_line_with_builtin_command(self):
        """Test executing built-in command"""
        registry = CommandRegistry()
        shell = InteractiveShell(registry)
        
        with patch('builtins.print') as mock_print:
            await shell._execute_line("help")
            
            mock_print.assert_called()
    
    @pytest.mark.asyncio
    async def test_execute_line_with_empty_input(self):
        """Test executing empty line"""
        registry = CommandRegistry()
        shell = InteractiveShell(registry)
        
        # Should not raise any exception
        await shell._execute_line("")
        await shell._execute_line("   ")
    
    @pytest.mark.asyncio
    async def test_execute_line_with_unknown_command(self):
        """Test executing unknown command"""
        registry = CommandRegistry()
        shell = InteractiveShell(registry)
        
        with patch('builtins.print') as mock_print:
            await shell._execute_line("unknown_command")
            
            mock_print.assert_called()
            # Should print error message
            calls = [call.args[0] for call in mock_print.call_args_list]
            error_text = ' '.join(str(call) for call in calls)
            assert "未知命令" in error_text


class TestCLIEdgeCases:
    """Test CLI edge cases and error conditions"""
    
    @pytest.mark.asyncio
    async def test_cli_app_with_missing_config(self):
        """Test CLI app behavior with missing configuration"""
        with patch('layers.infrastructure.config.config_manager.ConfigManager') as mock_config:
            mock_config.side_effect = Exception("Config not found")
            
            # Should handle config initialization error gracefully
            try:
                app = ADCCLIApp()
                # Should not raise exception during initialization
                assert hasattr(app, 'command_registry')
            except Exception:
                pytest.fail("CLI app should handle missing config gracefully")
    
    def test_command_registry_with_invalid_subparsers(self):
        """Test command registry with invalid subparsers"""
        registry = CommandRegistry()
        
        # Should handle None subparsers gracefully
        try:
            registry.register_commands(None)
        except Exception as e:
            # Should raise AttributeError or similar, which is expected
            assert isinstance(e, (AttributeError, TypeError))
    
    @pytest.mark.asyncio
    async def test_interactive_shell_with_malformed_input(self):
        """Test interactive shell with malformed input"""
        registry = CommandRegistry()
        shell = InteractiveShell(registry)
        
        # Test with various malformed inputs
        malformed_inputs = [
            "agent --invalid-flag-without-command",
            "workflow create --name",  # Missing value
            "team create --name 'unclosed quote",
            "system info --unknown-flag"
        ]
        
        for malformed_input in malformed_inputs:
            # Execution might fail, but should not crash
            with patch('builtins.print'):  # Suppress error output
                await shell._execute_line(malformed_input)
                # Should complete without raising exception
    
    def test_cli_app_with_unicode_arguments(self):
        """Test CLI app with unicode arguments"""
        app = ADCCLIApp()
        parser = app.create_parser()
        
        # Should handle unicode in arguments
        try:
            # This might raise SystemExit due to unknown command, but should not crash on unicode
            parser.parse_args(['--help'])
        except SystemExit:
            # Expected for help command
            pass
    
    @pytest.mark.asyncio
    async def test_command_execution_with_network_timeout(self):
        """Test command execution with network timeout"""
        registry = CommandRegistry()
        
        args = Mock()
        args.command = 'demo'
        args.demo_type = 'openai'
        
        # Mock network timeout
        with patch('layers.application.cli.commands.OpenAIAdapter') as mock_adapter:
            mock_adapter.side_effect = TimeoutError("Network timeout")
            
            with patch('builtins.print') as mock_print:
                result = await registry.execute_command(args)
                
                # Should handle timeout gracefully and return 0 (demo commands handle errors)
                assert result == 0
                mock_print.assert_called()


if __name__ == "__main__":
    pytest.main([__file__]) 


"""
适配器注册表
管理所有适配器的注册和发现
"""

import importlib
import logging
from typing import Dict, Any, List, Optional, Type, Set
from dataclasses import dataclass, field
from datetime import datetime

from ..base import BaseAdapter, AdapterStatus, AdapterCapability, AdapterError
from ...framework.abstractions.layer_communication import LayerMessage, LayerMessageType, LayerMessageStatus

logger = logging.getLogger(__name__)


@dataclass
class AdapterInfo:
    """适配器信息"""
    name: str
    adapter_class: Type[BaseAdapter]
    module_path: str
    capabilities: List[AdapterCapability]
    is_builtin: bool = False
    is_enabled: bool = True
    priority: int = 0
    metadata: Dict[str, Any] = field(default_factory=dict)


class AdapterRegistry:
    """
    适配器注册表
    
    负责管理所有适配器的注册、发现和生命周期管理
    """
    
    def __init__(self):
        self._adapters: Dict[str, AdapterInfo] = {}
        self._instances: Dict[str, BaseAdapter] = {}
        self._default_adapter: Optional[str] = None
        self._discovery_paths: List[str] = []
        
    def register_adapter(self, 
                        name: str, 
                        adapter_class: Type[BaseAdapter], 
                        capabilities: List[AdapterCapability],
                        module_path: str,
                        is_builtin: bool = False,
                        priority: int = 0,
                        metadata: Optional[Dict[str, Any]] = None) -> None:
        """
        注册适配器
        
        Args:
            name: 适配器名称
            adapter_class: 适配器类
            capabilities: 支持的能力
            module_path: 模块路径
            is_builtin: 是否为内置适配器
            priority: 优先级
            metadata: 额外元数据
        """
        if name in self._adapters:
            logger.warning(f"Adapter '{name}' already registered, overwriting")
        
        self._adapters[name] = AdapterInfo(
            name=name,
            adapter_class=adapter_class,
            module_path=module_path,
            capabilities=capabilities,
            is_builtin=is_builtin,
            priority=priority,
            metadata=metadata or {}
        )
        
        logger.info(f"Registered adapter: {name} (builtin: {is_builtin}, capabilities: {[c.value for c in capabilities]})")
    
    def unregister_adapter(self, name: str) -> None:
        """
        注销适配器
        
        Args:
            name: 适配器名称
        """
        if name in self._adapters:
            del self._adapters[name]
            logger.info(f"Unregistered adapter: {name}")
        
        if name in self._instances:
            del self._instances[name]
    
    async def get_adapter(self, 
                         name: str, 
                         config: Optional[Dict[str, Any]] = None,
                         force_recreate: bool = False) -> BaseAdapter:
        """
        获取适配器实例
        
        Args:
            name: 适配器名称
            config: 配置信息
            force_recreate: 是否强制重新创建
            
        Returns:
            BaseAdapter: 适配器实例
            
        Raises:
            AdapterError: 适配器不存在或初始化失败
        """
        if name not in self._adapters:
            raise AdapterError(f"Adapter '{name}' not found")
        
        adapter_info = self._adapters[name]
        
        # 检查适配器是否已存在且不需要重新创建
        if not force_recreate and name in self._instances:
            adapter = self._instances[name]
            if adapter.is_ready():
                return adapter
        
        # 创建新的适配器实例
        try:
            adapter = adapter_info.adapter_class(name)
            
            # 初始化适配器
            if config:
                await adapter.initialize(config)
            else:
                # 使用默认配置初始化
                await adapter.initialize({})
            
            self._instances[name] = adapter
            logger.info(f"Created and initialized adapter instance: {name}")
            
            return adapter
            
        except Exception as e:
            logger.error(f"Failed to create adapter '{name}': {str(e)}")
            raise AdapterError(f"Adapter creation failed: {str(e)}")
    
    def list_adapters(self, 
                     enabled_only: bool = True,
                     with_capabilities: Optional[List[AdapterCapability]] = None) -> List[str]:
        """
        列出所有适配器
        
        Args:
            enabled_only: 是否只列出启用的适配器
            with_capabilities: 过滤具有指定能力的适配器
            
        Returns:
            List[str]: 适配器名称列表
        """
        adapters = []
        
        for name, info in self._adapters.items():
            # 过滤启用的适配器
            if enabled_only and not info.is_enabled:
                continue
            
            # 过滤能力
            if with_capabilities:
                has_all_capabilities = all(
                    capability in info.capabilities for capability in with_capabilities
                )
                if not has_all_capabilities:
                    continue
            
            adapters.append(name)
        
        # 按优先级排序
        adapters.sort(key=lambda name: self._adapters[name].priority, reverse=True)
        
        return adapters
    
    def get_adapter_info(self, name: str) -> Optional[Dict[str, Any]]:
        """
        获取适配器详细信息
        
        Args:
            name: 适配器名称
            
        Returns:
            Optional[Dict[str, Any]]: 适配器信息，如果不存在返回None
        """
        if name not in self._adapters:
            return None
        
        info = self._adapters[name]
        return {
            "name": info.name,
            "class": info.adapter_class.__name__,
            "module": info.module_path,
            "capabilities": [cap.value for cap in info.capabilities],
            "is_builtin": info.is_builtin,
            "is_enabled": info.is_enabled,
            "priority": info.priority,
            "metadata": info.metadata
        }
    
    def set_default_adapter(self, name: str) -> None:
        """
        设置默认适配器
        
        Args:
            name: 适配器名称
            
        Raises:
            AdapterError: 适配器不存在
        """
        if name not in self._adapters:
            raise AdapterError(f"Adapter '{name}' not found")
        
        self._default_adapter = name
        logger.info(f"Set default adapter to: {name}")
    
    def get_default_adapter(self) -> Optional[str]:
        """
        获取默认适配器
        
        Returns:
            Optional[str]: 默认适配器名称
        """
        return self._default_adapter
    
    def enable_adapter(self, name: str) -> None:
        """
        启用适配器
        
        Args:
            name: 适配器名称
            
        Raises:
            AdapterError: 适配器不存在
        """
        if name not in self._adapters:
            raise AdapterError(f"Adapter '{name}' not found")
        
        self._adapters[name].is_enabled = True
        logger.info(f"Enabled adapter: {name}")
    
    def disable_adapter(self, name: str) -> None:
        """
        禁用适配器
        
        Args:
            name: 适配器名称
            
        Raises:
            AdapterError: 适配器不存在
        """
        if name not in self._adapters:
            raise AdapterError(f"Adapter '{name}' not found")
        
        self._adapters[name].is_enabled = False
        
        # 移除实例
        if name in self._instances:
            del self._instances[name]
        
        logger.info(f"Disabled adapter: {name}")
    
    def set_discovery_paths(self, paths: List[str]) -> None:
        """
        设置适配器发现路径
        
        Args:
            paths: 路径列表
        """
        self._discovery_paths = paths.copy()
        logger.info(f"Set discovery paths: {paths}")
    
    async def discover_adapters(self) -> List[str]:
        """
        自动发现适配器
        
        Returns:
            List[str]: 发现的适配器名称列表
        """
        discovered = []
        
        for path in self._discovery_paths:
            try:
                # 尝试导入模块并查找适配器类
                module = importlib.import_module(path)
                
                for attr_name in dir(module):
                    attr = getattr(module, attr_name)
                    
                    if (isinstance(attr, type) and 
                        issubclass(attr, BaseAdapter) and 
                        attr != BaseAdapter):
                        
                        # 获取适配器名称
                        adapter_name = getattr(attr, 'DEFAULT_NAME', attr.__name__.lower())
                        
                        # 获取能力列表
                        capabilities = getattr(attr, 'DEFAULT_CAPABILITIES', [])
                        
                        # 注册适配器
                        self.register_adapter(
                            name=adapter_name,
                            adapter_class=attr,
                            capabilities=capabilities,
                            module_path=path,
                            is_builtin=False
                        )
                        
                        discovered.append(adapter_name)
                        logger.info(f"Discovered adapter: {adapter_name} from {path}")
                        
            except ImportError as e:
                logger.warning(f"Failed to import discovery path {path}: {str(e)}")
                continue
        
        return discovered
    
    async def health_check_all(self) -> Dict[str, Dict[str, Any]]:
        """
        检查所有适配器的健康状态
        
        Returns:
            Dict[str, Dict[str, Any]]: 健康检查结果
        """
        results = {}
        
        for name in self.list_adapters(enabled_only=True):
            try:
                adapter = await self.get_adapter(name)
                health = await adapter.health_check()
                results[name] = health
            except Exception as e:
                results[name] = {
                    "status": "error",
                    "error": str(e)
                }
        
        return results
    
    def clear_instances(self) -> None:
        """清除所有适配器实例"""
        self._instances.clear()
        logger.info("Cleared all adapter instances")
    
    def __contains__(self, name: str) -> bool:
        """检查适配器是否存在"""
        return name in self._adapters
    
    def __len__(self) -> int:
        """获取适配器数量"""
        return len(self._adapters)
    
    def __str__(self) -> str:
        return f"AdapterRegistry(adapters={len(self._adapters)}, instances={len(self._instances)})"
    
    def __repr__(self) -> str:
        return self.__str__()


# 全局适配器注册表实例
_adapter_registry = AdapterRegistry()

def get_adapter_registry() -> AdapterRegistry:
    """获取全局适配器注册表"""
    return _adapter_registry

def register_adapter(name: str, 
                    adapter_class: Type[BaseAdapter], 
                    capabilities: List[AdapterCapability],
                    module_path: str,
                    **kwargs) -> None:
    """全局注册适配器"""
    _adapter_registry.register_adapter(
        name, adapter_class, capabilities, module_path, **kwargs
    )

async def get_adapter(name: str, **kwargs) -> BaseAdapter:
    """全局获取适配器"""
    return await _adapter_registry.get_adapter(name, **kwargs)

def list_adapters(**kwargs) -> List[str]:
    """全局列出适配器"""
    return _adapter_registry.list_adapters(**kwargs)



"""
KnowledgeBasedAgent基类
提供知识库驱动的Agent开发架构

设计理念：
- 知识库优先：80%的时间构建知识库，20%的时间写代码
- 声明式能力：通过装饰器声明Agent能力
- 智能增强：自动知识检索和上下文感知
- 可扩展性：支持多种知识源和AI后端
"""

import asyncio
import logging
from abc import ABC, abstractmethod
from typing import Dict, List, Any, Optional, Type, Union
from dataclasses import dataclass, field
from datetime import datetime
from pathlib import Path

from .decorators import (
    CapabilityMetadata, KnowledgeEnhancementConfig, ContextAwarenessConfig,
    list_all_capabilities, get_method_capabilities
)
from ...intelligent_context.integrated_knowledge_service import IntegratedKnowledgeService
from ...intelligent_context.context_engineering import ContextManager

logger = logging.getLogger(__name__)


@dataclass
class AgentRequest:
    """Agent请求"""
    content: str
    request_type: str = "general"
    metadata: Dict[str, Any] = field(default_factory=dict)
    context: Dict[str, Any] = field(default_factory=dict)
    user_id: Optional[str] = None
    session_id: Optional[str] = None
    created_at: datetime = field(default_factory=datetime.now)


@dataclass
class AgentResponse:
    """Agent响应"""
    content: str
    response_type: str = "text"
    metadata: Dict[str, Any] = field(default_factory=dict)
    confidence: float = 1.0
    processing_time: float = 0.0
    knowledge_sources: List[str] = field(default_factory=list)
    capabilities_used: List[str] = field(default_factory=list)
    created_at: datetime = field(default_factory=datetime.now)


@dataclass
class AgentConfig:
    """Agent配置"""
    name: str
    version: str = "1.0.0"
    description: str = ""
    knowledge_base_path: Optional[str] = None
    ai_backend: str = "mock"
    enable_knowledge_enhancement: bool = True
    enable_context_awareness: bool = True
    enable_performance_monitoring: bool = True
    max_context_window: int = 4000
    default_confidence_threshold: float = 0.7


class KnowledgeBasedAgent(ABC):
    """
    知识库驱动的Agent基类
    
    提供：
    - 自动知识库集成
    - 装饰器驱动的能力管理
    - 上下文感知和记忆
    - 性能监控和优化
    - 可扩展的AI后端支持
    """
    
    def __init__(self, config: AgentConfig):
        """初始化知识库驱动的Agent"""
        self.config = config
        self.name = config.name
        self.version = config.version
        self.agent_id = f"{config.name.lower()}_{datetime.now().strftime('%Y%m%d_%H%M%S')}"
        
        # 知识服务
        self.knowledge_service: Optional[IntegratedKnowledgeService] = None
        self.context_manager: Optional[ContextManager] = None
        
        # 状态管理
        self.is_initialized = False
        self.capabilities: Dict[str, CapabilityMetadata] = {}
        self.performance_stats = {
            'requests_processed': 0,
            'total_processing_time': 0.0,
            'average_processing_time': 0.0,
            'knowledge_retrieval_count': 0,
            'context_usage_count': 0
        }
        
        # 增强上下文（由装饰器注入）
        self.enhanced_context: Optional[List[Dict[str, Any]]] = None
        self.current_context: Optional[Dict[str, Any]] = None
        
        logger.info(f"🚀 创建知识库驱动Agent: {self.name} v{self.version}")
    
    async def initialize(self):
        """初始化Agent"""
        if self.is_initialized:
            logger.warning(f"Agent {self.name} 已经初始化")
            return
        
        logger.info(f"🔧 初始化Agent: {self.name}")
        
        try:
            # 1. 初始化知识服务
            if self.config.enable_knowledge_enhancement:
                await self._initialize_knowledge_service()
            
            # 2. 初始化上下文管理
            if self.config.enable_context_awareness:
                await self._initialize_context_manager()
            
            # 3. 发现和注册能力
            await self._discover_capabilities()
            
            # 4. 加载知识库
            if self.config.knowledge_base_path:
                await self._load_knowledge_base()
            
            # 5. 执行自定义初始化
            await self._custom_initialize()
            
            self.is_initialized = True
            logger.info(f"✅ Agent初始化完成: {self.name} ({len(self.capabilities)} 个能力)")
            
        except Exception as e:
            logger.error(f"❌ Agent初始化失败: {e}")
            raise
    
    async def _initialize_knowledge_service(self):
        """初始化知识服务"""
        try:
            self.knowledge_service = IntegratedKnowledgeService()
            await self.knowledge_service.initialize()
            logger.info("✅ 知识服务初始化成功")
        except Exception as e:
            logger.error(f"❌ 知识服务初始化失败: {e}")
            raise
    
    async def _initialize_context_manager(self):
        """初始化上下文管理器"""
        try:
            self.context_manager = ContextManager()
            await self.context_manager.initialize()
            logger.info("✅ 上下文管理器初始化成功")
        except Exception as e:
            logger.error(f"❌ 上下文管理器初始化失败: {e}")
            raise
    
    async def _discover_capabilities(self):
        """发现和注册Agent能力"""
        logger.info("🔍 发现Agent能力...")
        
        # 获取类的所有方法
        for attr_name in dir(self):
            attr = getattr(self, attr_name)
            
            # 检查是否为能力方法
            if hasattr(attr, '_is_capability') and attr._is_capability:
                capability_metadata = attr._capability_metadata
                self.capabilities[capability_metadata.name] = capability_metadata
                logger.debug(f"📋 发现能力: {capability_metadata.name} ({capability_metadata.capability_type.value})")
        
        logger.info(f"✅ 能力发现完成，共 {len(self.capabilities)} 个能力")
    
    async def _load_knowledge_base(self):
        """加载知识库"""
        if not self.knowledge_service or not self.config.knowledge_base_path:
            return
        
        logger.info(f"📚 加载知识库: {self.config.knowledge_base_path}")
        
        try:
            knowledge_path = Path(self.config.knowledge_base_path)
            if knowledge_path.exists():
                # 加载知识库文件
                await self._load_knowledge_files(knowledge_path)
                logger.info("✅ 知识库加载完成")
            else:
                logger.warning(f"知识库路径不存在: {knowledge_path}")
        except Exception as e:
            logger.error(f"❌ 知识库加载失败: {e}")
    
    async def _load_knowledge_files(self, knowledge_path: Path):
        """加载知识库文件"""
        # 这里可以实现具体的知识库加载逻辑
        # 支持多种格式：markdown, yaml, json等
        pass
    
    @abstractmethod
    async def _custom_initialize(self):
        """自定义初始化逻辑（子类实现）"""
        pass
    
    async def process_request(self, request: Union[str, AgentRequest]) -> AgentResponse:
        """处理请求"""
        if not self.is_initialized:
            await self.initialize()
        
        # 标准化请求
        if isinstance(request, str):
            request = AgentRequest(content=request)
        
        start_time = datetime.now()
        logger.info(f"📨 处理请求: {request.content[:100]}...")
        
        try:
            # 更新统计信息
            self.performance_stats['requests_processed'] += 1
            
            # 处理请求
            response = await self._process_request_internal(request)
            
            # 计算处理时间
            processing_time = (datetime.now() - start_time).total_seconds()
            response.processing_time = processing_time
            
            # 更新性能统计
            self._update_performance_stats(processing_time)
            
            logger.info(f"✅ 请求处理完成 (耗时: {processing_time:.2f}s, 置信度: {response.confidence:.2f})")
            return response
            
        except Exception as e:
            processing_time = (datetime.now() - start_time).total_seconds()
            logger.error(f"❌ 请求处理失败 (耗时: {processing_time:.2f}s): {e}")
            
            return AgentResponse(
                content=f"处理请求时发生错误: {str(e)}",
                response_type="error",
                confidence=0.0,
                processing_time=processing_time
            )
    
    @abstractmethod
    async def _process_request_internal(self, request: AgentRequest) -> AgentResponse:
        """内部请求处理逻辑（子类实现）"""
        pass
    
    async def _retrieve_knowledge(self, query: str, config: KnowledgeEnhancementConfig) -> List[Dict[str, Any]]:
        """检索相关知识"""
        if not self.knowledge_service:
            return []
        
        try:
            logger.debug(f"🔍 检索知识: {query[:50]}...")
            
            results = await self.knowledge_service.search(
                query=query,
                top_k=config.retrieval_count,
                confidence_threshold=config.confidence_threshold
            )
            
            # 更新统计信息
            self.performance_stats['knowledge_retrieval_count'] += 1
            
            # 转换为标准格式
            knowledge_items = []
            for result in results:
                knowledge_items.append({
                    'content': result.content,
                    'metadata': result.metadata,
                    'score': result.score,
                    'source': 'knowledge_base'
                })
            
            logger.debug(f"💡 检索到 {len(knowledge_items)} 条知识")
            return knowledge_items
            
        except Exception as e:
            logger.error(f"❌ 知识检索失败: {e}")
            return []
    
    async def _load_context(self, config: ContextAwarenessConfig) -> Dict[str, Any]:
        """加载上下文"""
        if not self.context_manager:
            return {}
        
        try:
            context = {}
            
            # 加载对话历史
            if config.enable_conversation_history:
                history = await self.context_manager.get_conversation_history(
                    window_size=config.history_window_size
                )
                context['history'] = history
            
            # 加载任务上下文
            if config.enable_task_context:
                task_context = await self.context_manager.get_task_context()
                context['task'] = task_context
            
            # 更新统计信息
            self.performance_stats['context_usage_count'] += 1
            
            return context
            
        except Exception as e:
            logger.error(f"❌ 上下文加载失败: {e}")
            return {}
    
    async def _update_context(self, method_name: str, args: tuple, kwargs: dict, result: Any):
        """更新上下文"""
        if not self.context_manager:
            return
        
        try:
            await self.context_manager.update_context({
                'method': method_name,
                'args': str(args)[:200],  # 限制长度
                'kwargs': str(kwargs)[:200],
                'result': str(result)[:500],
                'timestamp': datetime.now()
            })
        except Exception as e:
            logger.error(f"❌ 上下文更新失败: {e}")
    
    def _update_performance_stats(self, processing_time: float):
        """更新性能统计"""
        self.performance_stats['total_processing_time'] += processing_time
        self.performance_stats['average_processing_time'] = (
            self.performance_stats['total_processing_time'] / 
            self.performance_stats['requests_processed']
        )
    
    def get_capabilities(self) -> Dict[str, CapabilityMetadata]:
        """获取Agent能力"""
        return self.capabilities.copy()
    
    def get_performance_stats(self) -> Dict[str, Any]:
        """获取性能统计"""
        return self.performance_stats.copy()
    
    def get_agent_info(self) -> Dict[str, Any]:
        """获取Agent信息"""
        return {
            'name': self.name,
            'version': self.version,
            'agent_id': self.agent_id,
            'is_initialized': self.is_initialized,
            'capabilities_count': len(self.capabilities),
            'config': {
                'ai_backend': self.config.ai_backend,
                'enable_knowledge_enhancement': self.config.enable_knowledge_enhancement,
                'enable_context_awareness': self.config.enable_context_awareness,
                'max_context_window': self.config.max_context_window
            },
            'performance_stats': self.get_performance_stats()
        }
    
    async def health_check(self) -> Dict[str, Any]:
        """健康检查"""
        health_status = {
            'status': 'healthy',
            'agent_info': self.get_agent_info(),
            'services': {}
        }
        
        try:
            # 检查知识服务
            if self.knowledge_service:
                kb_health = await self.knowledge_service.health_check()
                health_status['services']['knowledge_service'] = kb_health
            
            # 检查上下文管理器
            if self.context_manager:
                context_health = await self.context_manager.health_check()
                health_status['services']['context_manager'] = context_health
            
        except Exception as e:
            health_status['status'] = 'unhealthy'
            health_status['error'] = str(e)
        
        return health_status
    
    async def cleanup(self):
        """清理资源"""
        logger.info(f"🧹 清理Agent资源: {self.name}")
        
        try:
            # 清理知识服务
            if self.knowledge_service:
                await self.knowledge_service.cleanup()
            
            # 清理上下文管理器
            if self.context_manager:
                await self.context_manager.cleanup()
            
            # 执行自定义清理
            await self._custom_cleanup()
            
            self.is_initialized = False
            logger.info(f"✅ Agent资源清理完成: {self.name}")
            
        except Exception as e:
            logger.error(f"❌ Agent资源清理失败: {e}")
    
    async def _custom_cleanup(self):
        """自定义清理逻辑（子类实现）"""
        pass
    
    def __repr__(self) -> str:
        return f"KnowledgeBasedAgent(name='{self.name}', version='{self.version}', capabilities={len(self.capabilities)})" 


"""
Embedding服务
集成sentence-transformers，提供文本向量化能力
"""

import logging
import asyncio
from typing import Dict, List, Any, Optional, Union
from dataclasses import dataclass, field
from datetime import datetime
import hashlib
import pickle
import os
from pathlib import Path

import numpy as np
from sentence_transformers import SentenceTransformer
import torch

logger = logging.getLogger(__name__)


@dataclass
class EmbeddingConfig:
    """Embedding配置"""
    model_name: str = "all-MiniLM-L6-v2"  # 默认轻量级模型
    cache_dir: str = "./data/embeddings_cache"
    max_sequence_length: int = 512
    batch_size: int = 32
    device: str = "auto"  # auto, cpu, cuda
    normalize_embeddings: bool = True


@dataclass
class EmbeddingResult:
    """Embedding结果"""
    text: str
    embedding: List[float]
    model_name: str
    created_at: datetime = field(default_factory=datetime.now)
    metadata: Dict[str, Any] = field(default_factory=dict)


class EmbeddingService:
    """
    Embedding服务
    
    提供文本向量化功能，支持多种模型和缓存机制
    """
    
    # 支持的模型配置
    SUPPORTED_MODELS = {
        # 通用英文模型
        "all-MiniLM-L6-v2": {
            "description": "轻量级通用模型，384维",
            "dimensions": 384,
            "max_seq_length": 256,
            "language": "en",
            "size": "80MB"
        },
        "all-mpnet-base-v2": {
            "description": "高质量通用模型，768维",
            "dimensions": 768,
            "max_seq_length": 384,
            "language": "en", 
            "size": "420MB"
        },
        # 多语言模型
        "paraphrase-multilingual-MiniLM-L12-v2": {
            "description": "多语言轻量级模型，384维",
            "dimensions": 384,
            "max_seq_length": 128,
            "language": "multilingual",
            "size": "420MB"
        },
        "paraphrase-multilingual-mpnet-base-v2": {
            "description": "多语言高质量模型，768维", 
            "dimensions": 768,
            "max_seq_length": 256,
            "language": "multilingual",
            "size": "970MB"
        },
        # 中文优化模型
        "shibing624/text2vec-base-chinese": {
            "description": "中文优化模型，768维",
            "dimensions": 768,
            "max_seq_length": 256,
            "language": "zh",
            "size": "400MB"
        }
    }
    
    def __init__(self, config: EmbeddingConfig = None):
        """初始化Embedding服务"""
        self.config = config or EmbeddingConfig()
        
        # 确保缓存目录存在
        Path(self.config.cache_dir).mkdir(parents=True, exist_ok=True)
        
        # 初始化模型
        self.model = None
        self.current_model_name = None
        
        # 缓存
        self._embedding_cache = {}
        self._cache_file = os.path.join(self.config.cache_dir, "embedding_cache.pkl")
        self._load_cache()
        
        # 统计信息
        self.stats = {
            "embeddings_generated": 0,
            "cache_hits": 0,
            "cache_misses": 0,
            "total_texts_processed": 0,
            "average_embedding_time": 0.0
        }
        
        # 不在构造函数中自动加载模型，等待显式初始化
    
    async def initialize(self, model_name: str = None) -> bool:
        """初始化嵌入服务，加载指定模型"""
        model_name = model_name or self.config.model_name
        return await self._load_model(model_name)
    
    def _get_cache_key(self, text: str, model_name: str) -> str:
        """生成缓存键"""
        content = f"{model_name}:{text}"
        return hashlib.md5(content.encode()).hexdigest()
    
    def _load_cache(self):
        """加载缓存"""
        try:
            if os.path.exists(self._cache_file):
                with open(self._cache_file, 'rb') as f:
                    self._embedding_cache = pickle.load(f)
                logger.info(f"✅ 已加载 {len(self._embedding_cache)} 个缓存embedding")
        except Exception as e:
            logger.warning(f"加载缓存失败: {e}")
            self._embedding_cache = {}
    
    def _save_cache(self):
        """保存缓存"""
        try:
            with open(self._cache_file, 'wb') as f:
                pickle.dump(self._embedding_cache, f)
        except Exception as e:
            logger.warning(f"保存缓存失败: {e}")
    
    async def _load_model(self, model_name: str) -> bool:
        """加载模型"""
        try:
            if self.current_model_name == model_name and self.model is not None:
                return True
            
            logger.info(f"🔄 正在加载embedding模型: {model_name}")
            
            # 确定设备
            device = self.config.device
            if device == "auto":
                device = "cuda" if torch.cuda.is_available() else "cpu"
            
            # 加载模型
            self.model = SentenceTransformer(
                model_name,
                device=device,
                cache_folder=os.path.join(self.config.cache_dir, "models")
            )
            
            # 设置最大序列长度
            if hasattr(self.model, 'max_seq_length'):
                self.model.max_seq_length = self.config.max_sequence_length
            
            self.current_model_name = model_name
            
            logger.info(f"✅ 模型加载成功: {model_name} (设备: {device})")
            return True
            
        except Exception as e:
            logger.error(f"❌ 模型加载失败: {e}")
            return False
    
    async def embed_text(self, 
                        text: str, 
                        model_name: str = None,
                        use_cache: bool = True) -> EmbeddingResult:
        """
        对单个文本进行向量化
        
        Args:
            text: 输入文本
            model_name: 模型名称，如果不指定则使用默认模型
            use_cache: 是否使用缓存
            
        Returns:
            Embedding结果
        """
        if not text.strip():
            raise ValueError("输入文本不能为空")
        
        model_name = model_name or self.config.model_name
        
        # 检查缓存
        cache_key = self._get_cache_key(text, model_name)
        if use_cache and cache_key in self._embedding_cache:
            self.stats["cache_hits"] += 1
            cached_result = self._embedding_cache[cache_key]
            logger.debug(f"🎯 缓存命中: {text[:50]}...")
            return EmbeddingResult(
                text=text,
                embedding=cached_result["embedding"],
                model_name=model_name,
                metadata={"from_cache": True}
            )
        
        # 确保模型已加载
        if not await self._load_model(model_name):
            raise RuntimeError(f"无法加载模型: {model_name}")
        
        try:
            start_time = datetime.now()
            
            # 生成embedding
            embedding = self.model.encode(
                text,
                normalize_embeddings=self.config.normalize_embeddings,
                convert_to_tensor=False
            )
            
            # 转换为列表格式
            if isinstance(embedding, np.ndarray):
                embedding = embedding.tolist()
            
            processing_time = (datetime.now() - start_time).total_seconds()
            
            # 更新统计信息
            self.stats["embeddings_generated"] += 1
            self.stats["cache_misses"] += 1
            self.stats["total_texts_processed"] += 1
            
            # 更新平均处理时间
            total_time = (self.stats["average_embedding_time"] * (self.stats["embeddings_generated"] - 1) + processing_time)
            self.stats["average_embedding_time"] = total_time / self.stats["embeddings_generated"]
            
            # 创建结果
            result = EmbeddingResult(
                text=text,
                embedding=embedding,
                model_name=model_name,
                metadata={
                    "processing_time": processing_time,
                    "dimensions": len(embedding),
                    "from_cache": False
                }
            )
            
            # 缓存结果
            if use_cache:
                self._embedding_cache[cache_key] = {
                    "embedding": embedding,
                    "created_at": datetime.now().isoformat()
                }
                
                # 定期保存缓存
                if len(self._embedding_cache) % 100 == 0:
                    self._save_cache()
            
            logger.debug(f"✅ Embedding生成完成: {text[:50]}... ({processing_time:.3f}s)")
            return result
            
        except Exception as e:
            logger.error(f"❌ Embedding生成失败: {e}")
            raise
    
    async def embed_texts(self, 
                         texts: List[str], 
                         model_name: str = None,
                         use_cache: bool = True) -> List[EmbeddingResult]:
        """
        批量文本向量化
        
        Args:
            texts: 文本列表
            model_name: 模型名称
            use_cache: 是否使用缓存
            
        Returns:
            Embedding结果列表
        """
        if not texts:
            return []
        
        model_name = model_name or self.config.model_name
        results = []
        
        # 分离缓存命中和未命中的文本
        cached_results = {}
        texts_to_process = []
        
        if use_cache:
            for i, text in enumerate(texts):
                cache_key = self._get_cache_key(text, model_name)
                if cache_key in self._embedding_cache:
                    cached_results[i] = self._embedding_cache[cache_key]
                    self.stats["cache_hits"] += 1
                else:
                    texts_to_process.append((i, text))
                    self.stats["cache_misses"] += 1
        else:
            texts_to_process = list(enumerate(texts))
        
        # 处理未缓存的文本
        if texts_to_process:
            # 确保模型已加载
            if not await self._load_model(model_name):
                raise RuntimeError(f"无法加载模型: {model_name}")
            
            try:
                start_time = datetime.now()
                
                # 批量生成embedding
                batch_texts = [text for _, text in texts_to_process]
                embeddings = self.model.encode(
                    batch_texts,
                    batch_size=self.config.batch_size,
                    normalize_embeddings=self.config.normalize_embeddings,
                    convert_to_tensor=False,
                    show_progress_bar=len(batch_texts) > 10
                )
                
                processing_time = (datetime.now() - start_time).total_seconds()
                
                # 处理结果
                for (original_idx, text), embedding in zip(texts_to_process, embeddings):
                    if isinstance(embedding, np.ndarray):
                        embedding = embedding.tolist()
                    
                    # 缓存结果
                    if use_cache:
                        cache_key = self._get_cache_key(text, model_name)
                        self._embedding_cache[cache_key] = {
                            "embedding": embedding,
                            "created_at": datetime.now().isoformat()
                        }
                    
                    # 存储到结果中
                    cached_results[original_idx] = {
                        "embedding": embedding,
                        "processing_time": processing_time / len(texts_to_process)
                    }
                
                # 更新统计信息
                self.stats["embeddings_generated"] += len(texts_to_process)
                self.stats["total_texts_processed"] += len(texts_to_process)
                
                logger.info(f"✅ 批量生成 {len(texts_to_process)} 个embedding ({processing_time:.3f}s)")
                
            except Exception as e:
                logger.error(f"❌ 批量embedding生成失败: {e}")
                raise
        
        # 按原始顺序构建结果
        for i, text in enumerate(texts):
            cached_data = cached_results.get(i, {})
            results.append(EmbeddingResult(
                text=text,
                embedding=cached_data["embedding"],
                model_name=model_name,
                metadata={
                    "processing_time": cached_data.get("processing_time", 0),
                    "dimensions": len(cached_data["embedding"]),
                    "from_cache": i not in [idx for idx, _ in texts_to_process]
                }
            ))
        
        # 保存缓存
        if use_cache and texts_to_process:
            self._save_cache()
        
        return results
    
    async def get_similarity(self, 
                            text1: str, 
                            text2: str, 
                            model_name: str = None) -> float:
        """
        计算两个文本的相似度
        
        Args:
            text1: 第一个文本
            text2: 第二个文本  
            model_name: 模型名称
            
        Returns:
            相似度分数 (0-1)
        """
        # 生成embeddings
        results = await self.embed_texts([text1, text2], model_name)
        
        # 计算余弦相似度
        emb1 = np.array(results[0].embedding)
        emb2 = np.array(results[1].embedding)
        
        similarity = np.dot(emb1, emb2) / (np.linalg.norm(emb1) * np.linalg.norm(emb2))
        
        return float(similarity)
    
    def get_model_info(self, model_name: str = None) -> Dict[str, Any]:
        """获取模型信息"""
        model_name = model_name or self.config.model_name
        
        if model_name in self.SUPPORTED_MODELS:
            info = self.SUPPORTED_MODELS[model_name].copy()
            info["is_loaded"] = (self.current_model_name == model_name and self.model is not None)
            return info
        
        return {"error": f"不支持的模型: {model_name}"}
    
    def list_supported_models(self) -> Dict[str, Dict[str, Any]]:
        """列出所有支持的模型"""
        return self.SUPPORTED_MODELS.copy()
    
    async def switch_model(self, model_name: str) -> bool:
        """切换模型"""
        if model_name not in self.SUPPORTED_MODELS:
            logger.error(f"不支持的模型: {model_name}")
            return False
        
        success = await self._load_model(model_name)
        if success:
            self.config.model_name = model_name
            logger.info(f"✅ 已切换到模型: {model_name}")
        
        return success
    
    def get_stats(self) -> Dict[str, Any]:
        """获取统计信息"""
        return {
            **self.stats,
            "cache_size": len(self._embedding_cache),
            "current_model": self.current_model_name,
            "cache_hit_rate": self.stats["cache_hits"] / max(1, self.stats["cache_hits"] + self.stats["cache_misses"])
        }
    
    def clear_cache(self) -> int:
        """清空缓存"""
        cache_size = len(self._embedding_cache)
        self._embedding_cache.clear()
        
        # 删除缓存文件
        if os.path.exists(self._cache_file):
            os.remove(self._cache_file)
        
        logger.info(f"✅ 已清空 {cache_size} 个缓存embedding")
        return cache_size
    
    async def health_check(self) -> Dict[str, Any]:
        """健康检查"""
        try:
            # 测试embedding生成
            test_text = "This is a health check test."
            result = await self.embed_text(test_text, use_cache=False)
            
            return {
                "status": "healthy",
                "model_loaded": self.model is not None,
                "current_model": self.current_model_name,
                "test_embedding_dimensions": len(result.embedding),
                "cache_size": len(self._embedding_cache),
                "stats": self.get_stats()
            }
            
        except Exception as e:
            logger.error(f"健康检查失败: {e}")
            return {
                "status": "unhealthy",
                "error": str(e),
                "model_loaded": self.model is not None
            }


# 单例实例
_embedding_service_instance = None

def get_embedding_service(config: EmbeddingConfig = None) -> EmbeddingService:
    """获取Embedding服务单例实例"""
    global _embedding_service_instance
    if _embedding_service_instance is None:
        _embedding_service_instance = EmbeddingService(config)
    return _embedding_service_instance 


"""
Context Manager - 上下文管理器
负责统一管理和协调各种上下文信息
"""

import asyncio
import logging
from typing import Dict, Any, List, Optional, Union
from datetime import datetime, timedelta
from dataclasses import dataclass, field
from enum import Enum

from .context_store import ContextStore
from .relevance_engine import RelevanceEngine
from .memory_manager import MemoryManager
from .context_adapter import ContextAdapter
from .context_optimizer import ContextOptimizer

logger = logging.getLogger(__name__)


class ContextType(Enum):
    """上下文类型"""
    SYSTEM = "system"           # 系统上下文
    USER = "user"              # 用户上下文
    CONVERSATION = "conversation"  # 对话上下文
    TASK = "task"              # 任务上下文
    DOMAIN = "domain"          # 领域上下文
    TEMPORAL = "temporal"      # 时间上下文
    ENVIRONMENTAL = "environmental"  # 环境上下文


class ContextPriority(Enum):
    """上下文优先级"""
    CRITICAL = 1    # 关键上下文
    HIGH = 2        # 高优先级
    MEDIUM = 3      # 中等优先级
    LOW = 4         # 低优先级
    BACKGROUND = 5  # 背景上下文


@dataclass
class ContextItem:
    """上下文项"""
    id: str
    type: ContextType
    content: Any
    priority: ContextPriority
    relevance_score: float = 0.0
    created_at: datetime = field(default_factory=datetime.now)
    last_accessed: datetime = field(default_factory=datetime.now)
    access_count: int = 0
    metadata: Dict[str, Any] = field(default_factory=dict)
    
    def update_access(self):
        """更新访问信息"""
        self.last_accessed = datetime.now()
        self.access_count += 1


class ContextManager:
    """上下文管理器"""
    
    def __init__(self, max_context_size: int = 8000, relevance_threshold: float = 0.3):
        self.max_context_size = max_context_size
        self.relevance_threshold = relevance_threshold
        
        # 初始化子组件
        self.context_store = ContextStore()
        self.relevance_engine = RelevanceEngine()
        self.memory_manager = MemoryManager()
        self.context_adapter = ContextAdapter()
        self.context_optimizer = ContextOptimizer()
        
        self.logger = logging.getLogger(f"{self.__class__.__name__}")
        
    async def build_context(self, 
                          task_description: str,
                          user_id: str = None,
                          session_id: str = None,
                          agent_type: str = None,
                          additional_context: Dict[str, Any] = None) -> Dict[str, Any]:
        """
        构建完整的上下文
        
        Args:
            task_description: 任务描述
            user_id: 用户ID
            session_id: 会话ID
            agent_type: Agent类型
            additional_context: 额外上下文
            
        Returns:
            构建好的上下文字典
        """
        try:
            self.logger.info(f"Building context for task: {task_description[:100]}...")
            
            # 1. 收集各类上下文
            contexts = await self._collect_contexts(
                task_description, user_id, session_id, agent_type, additional_context
            )
            
            # 2. 计算相关性评分
            await self._calculate_relevance_scores(contexts, task_description)
            
            # 3. 过滤和排序上下文
            filtered_contexts = self._filter_and_sort_contexts(contexts)
            
            # 4. 优化上下文大小
            optimized_contexts = await self._optimize_context_size(filtered_contexts)
            
            # 5. 适配上下文格式
            formatted_context = await self.context_adapter.format_context(
                optimized_contexts, agent_type
            )
            
            # 6. 更新访问统计
            await self._update_access_statistics(optimized_contexts)
            
            self.logger.info(f"Context built successfully with {len(optimized_contexts)} items")
            return formatted_context
            
        except Exception as e:
            self.logger.error(f"Error building context: {str(e)}")
            # 返回基础上下文
            return await self._build_fallback_context(task_description, agent_type)
    
    async def _collect_contexts(self, 
                              task_description: str,
                              user_id: str,
                              session_id: str,
                              agent_type: str,
                              additional_context: Dict[str, Any]) -> List[ContextItem]:
        """收集各类上下文"""
        contexts = []
        
        # 系统上下文
        system_context = await self._get_system_context(agent_type)
        contexts.extend(system_context)
        
        # 用户上下文
        if user_id:
            user_context = await self._get_user_context(user_id)
            contexts.extend(user_context)
        
        # 会话上下文
        if session_id:
            conversation_context = await self._get_conversation_context(session_id)
            contexts.extend(conversation_context)
        
        # 任务相关上下文
        task_context = await self._get_task_context(task_description)
        contexts.extend(task_context)
        
        # 领域知识上下文
        domain_context = await self._get_domain_context(task_description, agent_type)
        contexts.extend(domain_context)
        
        # 时间上下文
        temporal_context = await self._get_temporal_context()
        contexts.extend(temporal_context)
        
        # 额外上下文
        if additional_context:
            extra_context = self._process_additional_context(additional_context)
            contexts.extend(extra_context)
        
        return contexts
    
    async def _get_system_context(self, agent_type: str) -> List[ContextItem]:
        """获取系统上下文"""
        system_contexts = []
        
        # Agent角色定义
        role_definition = await self.context_store.get_agent_role(agent_type)
        if role_definition:
            system_contexts.append(ContextItem(
                id=f"role_{agent_type}",
                type=ContextType.SYSTEM,
                content=role_definition,
                priority=ContextPriority.CRITICAL,
                metadata={"category": "role_definition"}
            ))
        
        # 系统规则和约束
        system_rules = await self.context_store.get_system_rules()
        if system_rules:
            system_contexts.append(ContextItem(
                id="system_rules",
                type=ContextType.SYSTEM,
                content=system_rules,
                priority=ContextPriority.HIGH,
                metadata={"category": "rules"}
            ))
        
        return system_contexts
    
    async def _get_user_context(self, user_id: str) -> List[ContextItem]:
        """获取用户上下文"""
        user_contexts = []
        
        # 用户偏好
        user_preferences = await self.memory_manager.get_user_preferences(user_id)
        if user_preferences:
            user_contexts.append(ContextItem(
                id=f"preferences_{user_id}",
                type=ContextType.USER,
                content=user_preferences,
                priority=ContextPriority.HIGH,
                metadata={"category": "preferences"}
            ))
        
        # 用户历史行为
        user_history = await self.memory_manager.get_user_history(user_id, limit=10)
        if user_history:
            user_contexts.append(ContextItem(
                id=f"history_{user_id}",
                type=ContextType.USER,
                content=user_history,
                priority=ContextPriority.MEDIUM,
                metadata={"category": "history"}
            ))
        
        return user_contexts
    
    async def _get_conversation_context(self, session_id: str) -> List[ContextItem]:
        """获取对话上下文"""
        conversation_contexts = []
        
        # 获取对话历史
        conversation_history = await self.memory_manager.get_conversation_history(
            session_id, limit=20
        )
        
        if conversation_history:
            conversation_contexts.append(ContextItem(
                id=f"conversation_{session_id}",
                type=ContextType.CONVERSATION,
                content=conversation_history,
                priority=ContextPriority.HIGH,
                metadata={"category": "conversation_history"}
            ))
        
        return conversation_contexts
    
    async def _get_task_context(self, task_description: str) -> List[ContextItem]:
        """获取任务相关上下文"""
        task_contexts = []
        
        # 相似任务经验
        similar_tasks = await self.memory_manager.find_similar_tasks(task_description, limit=5)
        if similar_tasks:
            task_contexts.append(ContextItem(
                id="similar_tasks",
                type=ContextType.TASK,
                content=similar_tasks,
                priority=ContextPriority.MEDIUM,
                metadata={"category": "similar_tasks"}
            ))
        
        return task_contexts
    
    async def _get_domain_context(self, task_description: str, agent_type: str) -> List[ContextItem]:
        """获取领域知识上下文"""
        domain_contexts = []
        
        # 领域特定知识
        domain_knowledge = await self.context_store.get_domain_knowledge(
            task_description, agent_type
        )
        
        if domain_knowledge:
            domain_contexts.append(ContextItem(
                id="domain_knowledge",
                type=ContextType.DOMAIN,
                content=domain_knowledge,
                priority=ContextPriority.MEDIUM,
                metadata={"category": "domain_knowledge"}
            ))
        
        return domain_contexts
    
    async def _get_temporal_context(self) -> List[ContextItem]:
        """获取时间上下文"""
        temporal_contexts = []
        
        # 当前时间信息
        current_time = datetime.now()
        time_info = {
            "current_datetime": current_time.isoformat(),
            "day_of_week": current_time.strftime("%A"),
            "time_of_day": self._get_time_of_day(current_time)
        }
        
        temporal_contexts.append(ContextItem(
            id="current_time",
            type=ContextType.TEMPORAL,
            content=time_info,
            priority=ContextPriority.LOW,
            metadata={"category": "current_time"}
        ))
        
        return temporal_contexts
    
    def _process_additional_context(self, additional_context: Dict[str, Any]) -> List[ContextItem]:
        """处理额外上下文"""
        contexts = []
        
        for key, value in additional_context.items():
            contexts.append(ContextItem(
                id=f"additional_{key}",
                type=ContextType.ENVIRONMENTAL,
                content={key: value},
                priority=ContextPriority.MEDIUM,
                metadata={"category": "additional", "key": key}
            ))
        
        return contexts
    
    async def _calculate_relevance_scores(self, contexts: List[ContextItem], task_description: str):
        """计算相关性评分"""
        for context_item in contexts:
            score = await self.relevance_engine.calculate_relevance(
                context_item.content, task_description, context_item.type
            )
            context_item.relevance_score = score
    
    def _filter_and_sort_contexts(self, contexts: List[ContextItem]) -> List[ContextItem]:
        """过滤和排序上下文"""
        # 过滤低相关性的上下文
        filtered_contexts = [
            ctx for ctx in contexts 
            if ctx.relevance_score >= self.relevance_threshold or 
               ctx.priority.value <= ContextPriority.HIGH.value
        ]
        
        # 按优先级和相关性排序
        filtered_contexts.sort(
            key=lambda x: (x.priority.value, -x.relevance_score)
        )
        
        return filtered_contexts
    
    async def _optimize_context_size(self, contexts: List[ContextItem]) -> List[ContextItem]:
        """优化上下文大小"""
        return await self.context_optimizer.optimize_size(contexts, self.max_context_size)
    
    async def _update_access_statistics(self, contexts: List[ContextItem]):
        """更新访问统计"""
        for context_item in contexts:
            context_item.update_access()
            await self.context_store.update_context_stats(context_item)
    
    async def _build_fallback_context(self, task_description: str, agent_type: str) -> Dict[str, Any]:
        """构建回退上下文"""
        return {
            "system_prompt": f"You are a {agent_type} assistant.",
            "user_prompt": task_description,
            "context_items": [],
            "metadata": {"fallback": True}
        }
    
    def _get_time_of_day(self, current_time: datetime) -> str:
        """获取时段"""
        hour = current_time.hour
        if 5 <= hour < 12:
            return "morning"
        elif 12 <= hour < 17:
            return "afternoon"
        elif 17 <= hour < 21:
            return "evening"
        else:
            return "night"
    
    async def add_context_feedback(self, context_id: str, feedback: Dict[str, Any]):
        """添加上下文反馈"""
        await self.context_optimizer.add_feedback(context_id, feedback)
    
    async def get_context_analytics(self) -> Dict[str, Any]:
        """获取上下文分析"""
        return await self.context_store.get_analytics() 


#!/usr/bin/env python3
"""
高级融合策略演示

演示高级融合策略的核心功能：
1. 时间感知融合 - 基于信息新鲜度的融合
2. 语义融合 - 基于语义相似性的聚类融合
3. 成本感知融合 - 基于成本效益的优化融合
4. 质量驱动融合 - 基于权威性和质量的融合
5. 多视角融合 - 提供全面的多角度分析

Author: ADC Team
Date: 2024-12-19
Version: 1.0.0
"""

import asyncio
from datetime import datetime, timedelta
from typing import List, Dict, Any

def print_section(title: str):
    """打印章节标题"""
    print(f"\n{'='*70}")
    print(f"🎯 {title}")
    print(f"{'='*70}")

def print_subsection(title: str):
    """打印子章节标题"""
    print(f"\n{'─'*50}")
    print(f"📋 {title}")
    print(f"{'─'*50}")

async def demo_temporal_fusion():
    """演示时间感知融合"""
    print_section("时间感知融合演示")
    
    # 模拟不同新鲜度的知识源
    knowledge_sources = [
        {
            'source_type': 'local_kb',
            'content': 'FPGA是一种可编程逻辑器件，广泛应用于数字信号处理...',
            'confidence': 0.90,
            'cost': 0.1,
            'authority_level': 0.95,
            'freshness_score': 0.6,  # 较旧但权威
            'timestamp': datetime.now() - timedelta(days=30)
        },
        {
            'source_type': 'ai_training',
            'content': 'FPGA技术在2024年有了新的发展，特别是在AI加速领域...',
            'confidence': 0.85,
            'cost': 1.0,
            'authority_level': 0.80,
            'freshness_score': 0.8,  # 较新
            'timestamp': datetime.now() - timedelta(days=7)
        },
        {
            'source_type': 'web_search',
            'content': '最新的FPGA市场报告显示，2024年FPGA市场增长了15%...',
            'confidence': 0.75,
            'cost': 0.5,
            'authority_level': 0.70,
            'freshness_score': 0.95,  # 最新
            'timestamp': datetime.now() - timedelta(hours=2)
        }
    ]
    
    query = "FPGA的最新发展趋势是什么？"
    
    print(f"🔍 查询: {query}")
    print("\n📚 可用知识源:")
    
    for i, source in enumerate(knowledge_sources, 1):
        days_ago = (datetime.now() - source['timestamp']).days
        hours_ago = (datetime.now() - source['timestamp']).seconds // 3600
        
        time_desc = f"{days_ago}天前" if days_ago > 0 else f"{hours_ago}小时前"
        
        print(f"\n{i}. {source['source_type']} ({time_desc}):")
        print(f"   内容: {source['content'][:60]}...")
        print(f"   置信度: {source['confidence']:.2f}")
        print(f"   权威性: {source['authority_level']:.2f}")
        print(f"   新鲜度: {source['freshness_score']:.2f}")
        print(f"   成本: ${source['cost']:.2f}")
    
    print_subsection("时间感知融合分析")
    
    # 计算时间权重
    freshness_weight = 0.3
    print(f"🕐 新鲜度权重: {freshness_weight:.1%}")
    print(f"🏛️ 权威性权重: {1-freshness_weight:.1%}")
    
    weighted_scores = []
    for source in knowledge_sources:
        time_weight = source['freshness_score'] * freshness_weight
        authority_weight = source['authority_level'] * (1 - freshness_weight)
        total_weight = time_weight + authority_weight
        weighted_scores.append(total_weight)
        
        print(f"\n📊 {source['source_type']} 权重计算:")
        print(f"   时间权重: {source['freshness_score']:.2f} × {freshness_weight:.1%} = {time_weight:.3f}")
        print(f"   权威权重: {source['authority_level']:.2f} × {1-freshness_weight:.1%} = {authority_weight:.3f}")
        print(f"   总权重: {total_weight:.3f}")
    
    # 归一化权重
    total_weight = sum(weighted_scores)
    normalized_weights = [w / total_weight for w in weighted_scores]
    
    print_subsection("融合结果")
    
    print("🔄 时间感知融合内容:")
    total_cost = 0
    for i, (source, weight) in enumerate(zip(knowledge_sources, normalized_weights)):
        if weight > 0.1:  # 只显示权重显著的源
            freshness_label = "最新" if source['freshness_score'] > 0.8 else ("较新" if source['freshness_score'] > 0.5 else "较旧")
            print(f"\n【{freshness_label}信息 - 权重{weight:.1%}】")
            print(f"{source['content'][:80]}...")
            total_cost += source['cost']
    
    # 计算融合指标
    weighted_confidence = sum(source['confidence'] * weight 
                            for source, weight in zip(knowledge_sources, normalized_weights))
    
    print(f"\n📊 融合效果:")
    print(f"   融合置信度: {weighted_confidence:.3f}")
    print(f"   总成本: ${total_cost:.2f}")
    print(f"   时间平衡: 新鲜度与权威性的最优平衡")
    print(f"   信息完整性: 涵盖历史、现状、趋势三个时间维度")

async def demo_semantic_fusion():
    """演示语义融合"""
    print_section("语义融合演示")
    
    # 模拟不同语义角度的知识源
    knowledge_sources = [
        {
            'source_type': 'official_docs',
            'content': 'FPGA（现场可编程门阵列）的基本原理是通过配置存储器来定义逻辑功能...',
            'confidence': 0.95,
            'cost': 0.1,
            'relevance_score': 0.90,
            'semantic_category': '理论基础'
        },
        {
            'source_type': 'tutorial',
            'content': '学习FPGA设计的最佳实践包括：1. 掌握HDL语言 2. 理解时序概念...',
            'confidence': 0.85,
            'cost': 0.2,
            'relevance_score': 0.88,
            'semantic_category': '实践指南'
        },
        {
            'source_type': 'code_examples',
            'content': '以下是一个简单的FPGA计数器设计示例：\nmodule counter(clk, rst, count)...',
            'confidence': 0.80,
            'cost': 0.15,
            'relevance_score': 0.85,
            'semantic_category': '代码示例'
        },
        {
            'source_type': 'expert_knowledge',
            'content': 'FPGA设计中需要特别注意时序约束，建议使用流水线技术优化性能...',
            'confidence': 0.88,
            'cost': 0.8,
            'relevance_score': 0.92,
            'semantic_category': '专家经验'
        }
    ]
    
    query = "如何学习FPGA设计？"
    
    print(f"🔍 查询: {query}")
    print("\n📚 语义分析的知识源:")
    
    for i, source in enumerate(knowledge_sources, 1):
        print(f"\n{i}. {source['source_type']} - {source['semantic_category']}:")
        print(f"   内容: {source['content'][:60]}...")
        print(f"   置信度: {source['confidence']:.2f}")
        print(f"   相关性: {source['relevance_score']:.2f}")
        print(f"   成本: ${source['cost']:.2f}")
    
    print_subsection("语义聚类分析")
    
    # 按语义类别分组
    semantic_groups = {}
    for source in knowledge_sources:
        category = source['semantic_category']
        if category not in semantic_groups:
            semantic_groups[category] = []
        semantic_groups[category].append(source)
    
    print("🔍 语义聚类结果:")
    for category, sources in semantic_groups.items():
        avg_relevance = sum(s['relevance_score'] for s in sources) / len(sources)
        print(f"   📂 {category}: {len(sources)}个源, 平均相关性{avg_relevance:.2f}")
    
    print_subsection("语义融合结果")
    
    print("🔄 语义融合内容:")
    total_cost = 0
    source_contributions = {}
    
    for category, sources in semantic_groups.items():
        # 选择该类别中最佳代表
        best_source = max(sources, key=lambda x: x['confidence'] * x['relevance_score'])
        group_weight = sum(s['relevance_score'] for s in sources) / len(sources)
        
        print(f"\n【{category} - 相关性{group_weight:.1%}】")
        print(f"{best_source['content'][:100]}...")
        
        source_contributions[best_source['source_type']] = group_weight
        total_cost += best_source['cost']
    
    # 计算语义融合指标
    avg_relevance = sum(source['relevance_score'] for source in knowledge_sources) / len(knowledge_sources)
    semantic_coverage = len(semantic_groups)
    
    print(f"\n📊 语义融合效果:")
    print(f"   语义覆盖度: {semantic_coverage} 个主题维度")
    print(f"   平均相关性: {avg_relevance:.3f}")
    print(f"   内容多样性: {len(source_contributions)} 个不同源类型")
    print(f"   总成本: ${total_cost:.2f}")
    print(f"   融合优势: 提供了从理论到实践的完整学习路径")

async def demo_cost_aware_fusion():
    """演示成本感知融合"""
    print_section("成本感知融合演示")
    
    # 模拟不同成本效益的知识源
    knowledge_sources = [
        {
            'source_type': 'local_kb',
            'content': 'FPGA基础知识：可编程逻辑器件的工作原理...',
            'confidence': 0.85,
            'cost': 0.1,
            'efficiency': 8.5  # 置信度/成本
        },
        {
            'source_type': 'cached_response',
            'content': 'FPGA设计流程包括：需求分析、架构设计、RTL编码...',
            'confidence': 0.80,
            'cost': 0.001,  # 缓存成本极低
            'efficiency': 800  # 极高效益
        },
        {
            'source_type': 'ai_training',
            'content': '深度分析FPGA设计的关键技术要点和实现策略...',
            'confidence': 0.90,
            'cost': 1.2,
            'efficiency': 0.75  # 高质量但成本高
        },
        {
            'source_type': 'expert_consultation',
            'content': '专家建议：FPGA设计需要考虑的高级优化技巧...',
            'confidence': 0.95,
            'cost': 5.0,  # 专家咨询成本很高
            'efficiency': 0.19  # 质量最高但效益低
        }
    ]
    
    cost_budget = 2.0
    query = "FPGA设计的完整流程是什么？"
    
    print(f"🔍 查询: {query}")
    print(f"💰 成本预算: ${cost_budget:.2f}")
    print("\n📚 成本效益分析的知识源:")
    
    for i, source in enumerate(knowledge_sources, 1):
        print(f"\n{i}. {source['source_type']}:")
        print(f"   内容: {source['content'][:60]}...")
        print(f"   置信度: {source['confidence']:.2f}")
        print(f"   成本: ${source['cost']:.3f}")
        print(f"   效益比: {source['efficiency']:.1f} (置信度/成本)")
    
    print_subsection("成本感知融合分析")
    
    # 按效益排序
    sorted_sources = sorted(knowledge_sources, key=lambda x: x['efficiency'], reverse=True)
    
    print("📊 按成本效益排序:")
    for i, source in enumerate(sorted_sources, 1):
        efficiency_label = "极高效" if source['efficiency'] > 10 else ("高效" if source['efficiency'] > 1 else "低效")
        print(f"   {i}. {source['source_type']}: 效益{source['efficiency']:.1f} ({efficiency_label})")
    
    # 选择在预算内的最优组合
    selected_sources = []
    cumulative_cost = 0.0
    
    print(f"\n💡 在${cost_budget:.2f}预算内的最优选择:")
    
    for source in sorted_sources:
        if cumulative_cost + source['cost'] <= cost_budget:
            selected_sources.append(source)
            cumulative_cost += source['cost']
            print(f"   ✅ 选择 {source['source_type']}: +${source['cost']:.3f} (累计: ${cumulative_cost:.3f})")
        else:
            print(f"   ❌ 跳过 {source['source_type']}: ${source['cost']:.3f} 超预算")
    
    print_subsection("成本感知融合结果")
    
    if selected_sources:
        print("🔄 成本优化融合内容:")
        
        total_efficiency = sum(s['efficiency'] for s in selected_sources)
        source_contributions = {}
        
        for source in selected_sources:
            weight = source['efficiency'] / total_efficiency
            source_contributions[source['source_type']] = weight
            
            efficiency_label = "极高效" if source['efficiency'] > 10 else ("高效" if source['efficiency'] > 1 else "一般")
            print(f"\n【{efficiency_label}源 - 权重{weight:.1%}】")
            print(f"{source['content'][:80]}...")
        
        # 计算融合效果
        weighted_confidence = sum(source['confidence'] * source_contributions[source['source_type']] 
                                for source in selected_sources)
        
        print(f"\n📊 成本感知融合效果:")
        print(f"   融合置信度: {weighted_confidence:.3f}")
        print(f"   实际成本: ${cumulative_cost:.3f} / ${cost_budget:.2f}")
        print(f"   预算利用率: {cumulative_cost/cost_budget:.1%}")
        print(f"   平均效益: {sum(s['efficiency'] for s in selected_sources)/len(selected_sources):.1f}")
        print(f"   成本节省: ${sum(s['cost'] for s in knowledge_sources) - cumulative_cost:.2f}")
    
    else:
        print("❌ 预算不足，无法选择任何源")

async def demo_quality_driven_fusion():
    """演示质量驱动融合"""
    print_section("质量驱动融合演示")
    
    # 模拟不同质量等级的知识源
    knowledge_sources = [
        {
            'source_type': 'ieee_paper',
            'content': 'IEEE论文：FPGA架构优化的理论基础和数学模型...',
            'confidence': 0.95,
            'authority_level': 0.98,  # 极高权威性
            'cost': 2.0,
            'quality_score': 0.965
        },
        {
            'source_type': 'vendor_docs',
            'content': 'Xilinx官方文档：UltraScale架构的设计指南和最佳实践...',
            'confidence': 0.90,
            'authority_level': 0.95,  # 很高权威性
            'cost': 0.3,
            'quality_score': 0.925
        },
        {
            'source_type': 'community_wiki',
            'content': '社区Wiki：FPGA设计经验分享和常见问题解答...',
            'confidence': 0.75,
            'authority_level': 0.60,  # 中等权威性
            'cost': 0.05,
            'quality_score': 0.675
        },
        {
            'source_type': 'blog_post',
            'content': '技术博客：我的FPGA学习心得和项目经验分享...',
            'confidence': 0.70,
            'authority_level': 0.40,  # 较低权威性
            'cost': 0.02,
            'quality_score': 0.550
        }
    ]
    
    query = "FPGA架构优化的最佳方法？"
    user_role = "expert"  # 专家用户，重视质量
    
    print(f"🔍 查询: {query}")
    print(f"👤 用户角色: {user_role} (重视质量)")
    print("\n📚 不同质量等级的知识源:")
    
    for i, source in enumerate(knowledge_sources, 1):
        authority_label = "极高" if source['authority_level'] > 0.9 else ("很高" if source['authority_level'] > 0.8 else ("中等" if source['authority_level'] > 0.6 else "较低"))
        
        print(f"\n{i}. {source['source_type']} - {authority_label}权威性:")
        print(f"   内容: {source['content'][:60]}...")
        print(f"   置信度: {source['confidence']:.2f}")
        print(f"   权威性: {source['authority_level']:.2f}")
        print(f"   质量分数: {source['quality_score']:.3f}")
        print(f"   成本: ${source['cost']:.2f}")
    
    print_subsection("质量驱动融合分析")
    
    authority_weight = 0.4
    print(f"🏛️ 权威性权重: {authority_weight:.1%}")
    print(f"🎯 置信度权重: {1-authority_weight:.1%}")
    
    # 计算质量权重
    quality_weights = []
    for source in knowledge_sources:
        quality_score = (source['confidence'] * (1 - authority_weight) + 
                        source['authority_level'] * authority_weight)
        quality_weights.append(quality_score)
        
        print(f"\n📊 {source['source_type']} 质量权重:")
        print(f"   置信度贡献: {source['confidence']:.2f} × {1-authority_weight:.1%} = {source['confidence'] * (1-authority_weight):.3f}")
        print(f"   权威性贡献: {source['authority_level']:.2f} × {authority_weight:.1%} = {source['authority_level'] * authority_weight:.3f}")
        print(f"   质量分数: {quality_score:.3f}")
    
    # 归一化权重
    total_quality = sum(quality_weights)
    normalized_weights = [w / total_quality for w in quality_weights]
    
    print_subsection("质量驱动融合结果")
    
    print("🔄 质量驱动融合内容:")
    total_cost = 0
    source_contributions = {}
    
    # 按质量权重排序，只选择高质量源
    quality_ranked = list(zip(knowledge_sources, normalized_weights))
    quality_ranked.sort(key=lambda x: x[1], reverse=True)
    
    for source, weight in quality_ranked:
        if weight > 0.15:  # 只包含高权重源
            authority_label = "权威" if source['authority_level'] > 0.8 else ("可信" if source['authority_level'] > 0.6 else "一般")
            confidence_label = "高信度" if source['confidence'] > 0.8 else ("中信度" if source['confidence'] > 0.6 else "低信度")
            
            print(f"\n【{authority_label}源 - {confidence_label}, 权重{weight:.1%}】")
            print(f"{source['content'][:90]}...")
            
            source_contributions[source['source_type']] = weight
            total_cost += source['cost']
    
    # 计算质量指标
    included_sources = [source for source, weight in quality_ranked 
                       if source['source_type'] in source_contributions]
    avg_authority = sum(s['authority_level'] for s in included_sources) / len(included_sources)
    avg_confidence = sum(s['confidence'] for s in included_sources) / len(included_sources)
    
    print(f"\n📊 质量驱动融合效果:")
    print(f"   平均权威性: {avg_authority:.3f}")
    print(f"   平均置信度: {avg_confidence:.3f}")
    print(f"   质量一致性: {min(s['confidence'] for s in included_sources):.3f}")
    print(f"   总成本: ${total_cost:.2f}")
    print(f"   质量保证: 专家级用户获得最高质量的融合内容")

async def demo_multi_perspective_fusion():
    """演示多视角融合"""
    print_section("多视角融合演示")
    
    # 模拟多个不同视角的知识源
    knowledge_sources = [
        {
            'source_type': 'local_kb',
            'content': '从技术角度看，FPGA相比CPU具有并行处理优势，适合特定算法加速...',
            'confidence': 0.88,
            'cost': 0.1,
            'perspective': '技术视角'
        },
        {
            'source_type': 'ai_training',
            'content': '从商业角度分析，FPGA虽然开发成本高，但在特定应用中ROI显著...',
            'confidence': 0.82,
            'cost': 1.0,
            'perspective': '商业视角'
        },
        {
            'source_type': 'web_search',
            'content': '从市场趋势看，FPGA在AI推理、5G通信、自动驾驶等领域需求激增...',
            'confidence': 0.78,
            'cost': 0.5,
            'perspective': '市场视角'
        },
        {
            'source_type': 'academic_research',
            'content': '从学术研究角度，FPGA在量子计算、神经形态计算等前沿领域有重要应用...',
            'confidence': 0.85,
            'cost': 1.5,
            'perspective': '学术视角'
        }
    ]
    
    query = "FPGA相比CPU有什么优势？"
    
    print(f"🔍 查询: {query}")
    print("\n📚 多视角知识源:")
    
    for i, source in enumerate(knowledge_sources, 1):
        print(f"\n{i}. {source['perspective']} ({source['source_type']}):")
        print(f"   内容: {source['content'][:60]}...")
        print(f"   置信度: {source['confidence']:.2f}")
        print(f"   成本: ${source['cost']:.2f}")
    
    print_subsection("多视角融合分析")
    
    # 检查视角多样性
    perspectives = set(source['perspective'] for source in knowledge_sources)
    diversity_score = len(perspectives) / len(knowledge_sources)
    
    print(f"🔍 视角多样性分析:")
    print(f"   总视角数: {len(perspectives)}")
    print(f"   总源数: {len(knowledge_sources)}")
    print(f"   多样性分数: {diversity_score:.2f}")
    print(f"   多样性评估: {'优秀' if diversity_score > 0.8 else ('良好' if diversity_score > 0.6 else '一般')}")
    
    # 计算视角权重
    perspective_weights = {}
    for source in knowledge_sources:
        # 视角权重 = (置信度 + 权威性) / 2
        # 这里用置信度代替权威性进行演示
        perspective_weight = (source['confidence'] + source['confidence']) / 2
        perspective_weights[source['perspective']] = perspective_weight
    
    print("\n📊 视角权重分配:")
    total_perspective_weight = sum(perspective_weights.values())
    for perspective, weight in perspective_weights.items():
        normalized_weight = weight / total_perspective_weight
        print(f"   {perspective}: {normalized_weight:.1%}")
    
    print_subsection("多视角融合结果")
    
    print("🔄 多视角融合内容:")
    
    source_contributions = {}
    total_cost = 0
    
    for source in knowledge_sources:
        perspective_weight = perspective_weights[source['perspective']] / total_perspective_weight
        
        print(f"\n【{source['perspective']} - 权重{perspective_weight:.1%}】")
        print(f"{source['content'][:90]}...")
        
        source_contributions[source['source_type']] = perspective_weight
        total_cost += source['cost']
    
    # 计算多视角融合指标
    weighted_confidence = sum(source['confidence'] * perspective_weights[source['perspective']] / total_perspective_weight 
                            for source in knowledge_sources)
    
    viewpoint_balance = min(perspective_weights.values()) / max(perspective_weights.values()) if perspective_weights else 0
    
    print(f"\n📊 多视角融合效果:")
    print(f"   视角多样性: {len(perspectives)} 个不同视角")
    print(f"   视角平衡度: {viewpoint_balance:.3f}")
    print(f"   综合置信度: {weighted_confidence:.3f}")
    print(f"   总成本: ${total_cost:.2f}")
    print(f"   融合优势: 提供了技术、商业、市场、学术的全方位分析")

async def demo_intelligent_fusion_decision():
    """演示智能融合决策"""
    print_section("智能融合决策演示")
    
    # 模拟不同场景的融合决策
    test_scenarios = [
        {
            'query': '最新的FPGA技术发展趋势？',
            'user_role': 'researcher',
            'source_count': 3,
            'confidence_variance': 0.15,  # 置信度差异小
            'has_temporal_keywords': True,
            'expected_strategy': 'temporal_fusion'
        },
        {
            'query': '如何实现复杂的FPGA状态机设计？',
            'user_role': 'expert',
            'source_count': 4,
            'confidence_variance': 0.25,  # 置信度差异大
            'has_semantic_complexity': True,
            'expected_strategy': 'semantic_fusion'
        },
        {
            'query': 'FPGA设计入门指南',
            'user_role': 'beginner',
            'source_count': 3,
            'cost_budget': 1.0,  # 预算有限
            'cost_sensitive': True,
            'expected_strategy': 'cost_aware_fusion'
        },
        {
            'query': 'FPGA vs ASIC vs GPU的全面比较',
            'user_role': 'expert',
            'source_count': 4,
            'needs_multiple_perspectives': True,
            'expected_strategy': 'multi_perspective_fusion'
        }
    ]
    
    print("🧠 智能融合决策分析...")
    
    for i, scenario in enumerate(test_scenarios, 1):
        print(f"\n{i}. 场景分析:")
        print(f"   查询: {scenario['query']}")
        print(f"   用户角色: {scenario['user_role']}")
        print(f"   可用源数: {scenario['source_count']}")
        
        # 模拟决策过程
        decision_result = make_fusion_decision(scenario)
        
        print(f"   🎯 决策策略: {decision_result['strategy']}")
        print(f"   📊 置信度: {decision_result['confidence']:.3f}")
        print(f"   💰 预期成本: ${decision_result['expected_cost']:.2f}")
        print(f"   🔍 决策理由: {decision_result['reasoning']}")
        print(f"   ✅ 符合预期: {scenario['expected_strategy']}")
    
    print_subsection("融合策略统计")
    
    strategy_usage = {}
    for scenario in test_scenarios:
        decision = make_fusion_decision(scenario)
        strategy = decision['strategy']
        strategy_usage[strategy] = strategy_usage.get(strategy, 0) + 1
    
    print("📊 融合策略使用分布:")
    for strategy, count in strategy_usage.items():
        percentage = count / len(test_scenarios) * 100
        print(f"   {strategy.replace('_', ' ').title()}: {count} 次 ({percentage:.1f}%)")
    
    print(f"\n🎯 决策智能化效果:")
    print(f"   策略多样性: {len(strategy_usage)} 种不同策略")
    print(f"   决策准确性: 100% (所有决策符合预期)")
    print(f"   自适应能力: 根据查询特征自动选择最优策略")

def make_fusion_decision(scenario: Dict[str, Any]) -> Dict[str, Any]:
    """模拟智能融合决策"""
    
    # 时间感知融合触发条件
    if scenario.get('has_temporal_keywords', False):
        return {
            'strategy': 'temporal_fusion',
            'confidence': 0.85,
            'expected_cost': 1.6,
            'reasoning': '查询包含时间敏感关键词，选择时间感知融合'
        }
    
    # 语义融合触发条件
    if scenario.get('has_semantic_complexity', False):
        return {
            'strategy': 'semantic_fusion',
            'confidence': 0.88,
            'expected_cost': 2.2,
            'reasoning': '复杂语义查询，选择语义聚类融合'
        }
    
    # 成本感知融合触发条件
    if scenario.get('cost_sensitive', False):
        return {
            'strategy': 'cost_aware_fusion',
            'confidence': 0.80,
            'expected_cost': 0.8,
            'reasoning': '成本敏感用户，选择成本优化融合'
        }
    
    # 多视角融合触发条件
    if scenario.get('needs_multiple_perspectives', False):
        return {
            'strategy': 'multi_perspective_fusion',
            'confidence': 0.90,
            'expected_cost': 2.5,
            'reasoning': '需要多角度分析，选择多视角融合'
        }
    
    # 默认策略
    return {
        'strategy': 'quality_driven_fusion',
        'confidence': 0.82,
        'expected_cost': 1.2,
        'reasoning': '默认选择质量驱动融合'
    }

async def demo_fusion_performance_comparison():
    """演示融合性能对比"""
    print_section("融合性能对比演示")
    
    # 模拟不同融合策略的性能数据
    strategy_performance = {
        'temporal_fusion': {
            'avg_confidence': 0.83,
            'avg_cost': 1.6,
            'avg_latency': 2.8,
            'user_satisfaction': 0.87,
            'use_cases': ['时间敏感查询', '趋势分析', '最新信息需求']
        },
        'semantic_fusion': {
            'avg_confidence': 0.88,
            'avg_cost': 2.2,
            'avg_latency': 3.2,
            'user_satisfaction': 0.91,
            'use_cases': ['复杂概念查询', '技术对比', '深度分析']
        },
        'cost_aware_fusion': {
            'avg_confidence': 0.80,
            'avg_cost': 0.8,
            'avg_latency': 1.5,
            'user_satisfaction': 0.82,
            'use_cases': ['预算限制', '高频查询', '成本敏感场景']
        },
        'quality_driven_fusion': {
            'avg_confidence': 0.92,
            'avg_cost': 2.8,
            'avg_latency': 3.5,
            'user_satisfaction': 0.94,
            'use_cases': ['专家用户', '关键决策', '高质量要求']
        },
        'multi_perspective_fusion': {
            'avg_confidence': 0.86,
            'avg_cost': 2.5,
            'avg_latency': 4.0,
            'user_satisfaction': 0.89,
            'use_cases': ['对比分析', '全面评估', '决策支持']
        }
    }
    
    print("📊 融合策略性能对比:")
    
    print(f"\n{'策略':<20} {'置信度':<8} {'成本':<8} {'延迟':<8} {'满意度':<8}")
    print("-" * 60)
    
    for strategy, perf in strategy_performance.items():
        strategy_name = strategy.replace('_', ' ').title()[:18]
        print(f"{strategy_name:<20} {perf['avg_confidence']:<8.3f} ${perf['avg_cost']:<7.2f} {perf['avg_latency']:<8.1f}s {perf['user_satisfaction']:<8.1%}")
    
    print_subsection("策略适用场景")
    
    for strategy, perf in strategy_performance.items():
        strategy_name = strategy.replace('_', ' ').title()
        print(f"\n🎯 {strategy_name}:")
        print(f"   适用场景: {', '.join(perf['use_cases'])}")
        
        # 性能特点
        if perf['avg_cost'] < 1.0:
            print(f"   💰 成本优势: 低成本 (${perf['avg_cost']:.2f})")
        if perf['avg_latency'] < 2.0:
            print(f"   ⚡ 速度优势: 快速响应 ({perf['avg_latency']:.1f}s)")
        if perf['avg_confidence'] > 0.9:
            print(f"   🎯 质量优势: 高置信度 ({perf['avg_confidence']:.1%})")
        if perf['user_satisfaction'] > 0.9:
            print(f"   😊 体验优势: 高满意度 ({perf['user_satisfaction']:.1%})")
    
    print_subsection("融合策略选择建议")
    
    print("💡 智能选择建议:")
    print("   🕐 时间敏感查询 → temporal_fusion (新鲜度优先)")
    print("   🧠 复杂技术问题 → semantic_fusion (语义聚类)")
    print("   💰 预算限制场景 → cost_aware_fusion (成本优化)")
    print("   👨‍💼 专家用户查询 → quality_driven_fusion (质量保证)")
    print("   📊 对比分析需求 → multi_perspective_fusion (全面视角)")
    
    # 性能综合评估
    print("\n🏆 性能综合排名:")
    
    # 计算综合分数（置信度40% + 满意度30% + 成本效益20% + 速度10%）
    strategy_scores = {}
    for strategy, perf in strategy_performance.items():
        cost_efficiency = 1 / max(perf['avg_cost'], 0.1)  # 成本效益
        speed_score = 1 / max(perf['avg_latency'], 0.1)   # 速度分数
        
        # 归一化分数
        normalized_cost_efficiency = min(1.0, cost_efficiency / 10)
        normalized_speed = min(1.0, speed_score / 1)
        
        comprehensive_score = (
            perf['avg_confidence'] * 0.4 +
            perf['user_satisfaction'] * 0.3 +
            normalized_cost_efficiency * 0.2 +
            normalized_speed * 0.1
        )
        
        strategy_scores[strategy] = comprehensive_score
    
    ranked_strategies = sorted(strategy_scores.items(), key=lambda x: x[1], reverse=True)
    
    for i, (strategy, score) in enumerate(ranked_strategies, 1):
        strategy_name = strategy.replace('_', ' ').title()
        medal = "🥇" if i == 1 else ("🥈" if i == 2 else ("🥉" if i == 3 else "🏅"))
        print(f"   {medal} {i}. {strategy_name}: {score:.3f}")

async def main():
    """主演示函数"""
    print("🔄 高级融合策略完整演示")
    print("=" * 70)
    
    try:
        # 1. 时间感知融合
        await demo_temporal_fusion()
        
        # 2. 语义融合
        await demo_semantic_fusion()
        
        # 3. 成本感知融合
        await demo_cost_aware_fusion()
        
        # 4. 质量驱动融合
        await demo_quality_driven_fusion()
        
        # 5. 多视角融合
        await demo_multi_perspective_fusion()
        
        # 6. 智能融合决策
        await demo_intelligent_fusion_decision()
        
        # 7. 性能对比
        await demo_fusion_performance_comparison()
        
        print_section("演示总结")
        
        print("🎊 高级融合策略核心价值:")
        print("   ✅ 时间感知 - 智能平衡新鲜度与权威性")
        print("   ✅ 语义聚类 - 自动分组相关内容，避免重复")
        print("   ✅ 成本优化 - 在预算约束下获得最佳效益")
        print("   ✅ 质量保证 - 专家级用户获得最高质量内容")
        print("   ✅ 多视角 - 提供全面的多角度分析")
        print("   ✅ 智能决策 - 根据查询特征自动选择最优策略")
        
        print("\n💡 技术创新:")
        print("   🧠 自适应策略选择 - 基于查询特征智能匹配")
        print("   📊 多维度权重计算 - 综合考虑质量、成本、时间")
        print("   🔄 动态内容组织 - 智能聚类和分层展示")
        print("   🎯 个性化融合 - 基于用户角色的定制化策略")
        print("   📈 持续优化 - 基于历史数据的策略调优")
        
        print("\n🚀 商业价值:")
        print("   💰 成本控制: 智能预算管理，避免不必要的高成本查询")
        print("   🎯 质量保证: 专家用户获得权威性和准确性保证")
        print("   ⚡ 效率提升: 语义聚类避免信息重复，提高阅读效率")
        print("   🌐 全面性: 多视角融合提供更全面的信息覆盖")
        print("   🔮 前瞻性: 时间感知融合确保信息的时效性")
        
    except Exception as e:
        print(f"❌ 演示过程中出现错误: {e}")
        import traceback
        traceback.print_exc()

if __name__ == "__main__":
    asyncio.run(main()) 


"""
LLM Provider Base Classes
定义所有LLM提供商的统一接口和抽象基类
"""

import asyncio
import logging
from abc import ABC, abstractmethod
from typing import Dict, Any, List, Optional, Union, AsyncGenerator
from datetime import datetime
from enum import Enum
import uuid

logger = logging.getLogger(__name__)


class LLMModelType(Enum):
    """LLM模型类型枚举"""
    CHAT = "chat"              # 对话模型
    COMPLETION = "completion"  # 补全模型
    EMBEDDING = "embedding"    # 嵌入模型
    IMAGE = "image"           # 图像理解模型
    CODE = "code"             # 代码模型


class LLMCapability(Enum):
    """LLM能力枚举"""
    FUNCTION_CALLING = "function_calling"
    STREAMING = "streaming"
    VISION = "vision"
    JSON_MODE = "json_mode"
    SYSTEM_MESSAGE = "system_message"
    MULTI_TURN = "multi_turn"
    TOOL_USE = "tool_use"


class LLMRequest:
    """LLM请求封装"""
    
    def __init__(
        self,
        messages: List[Dict[str, str]],
        model: Optional[str] = None,
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
        stream: bool = False,
        functions: Optional[List[Dict[str, Any]]] = None,
        tools: Optional[List[Dict[str, Any]]] = None,
        response_format: Optional[Dict[str, Any]] = None,
        **kwargs
    ):
        self.request_id = str(uuid.uuid4())
        self.messages = messages
        self.model = model
        self.temperature = temperature
        self.max_tokens = max_tokens
        self.stream = stream
        self.functions = functions
        self.tools = tools
        self.response_format = response_format
        self.extra_params = kwargs
        self.created_at = datetime.now()


class LLMResponse:
    """LLM响应封装"""
    
    def __init__(
        self,
        request_id: str,
        content: str,
        model: str,
        usage: Optional[Dict[str, int]] = None,
        finish_reason: Optional[str] = None,
        function_call: Optional[Dict[str, Any]] = None,
        tool_calls: Optional[List[Dict[str, Any]]] = None,
        metadata: Optional[Dict[str, Any]] = None
    ):
        self.request_id = request_id
        self.response_id = str(uuid.uuid4())
        self.content = content
        self.model = model
        self.usage = usage or {}
        self.finish_reason = finish_reason
        self.function_call = function_call
        self.tool_calls = tool_calls
        self.metadata = metadata or {}
        self.created_at = datetime.now()
    
    def to_dict(self) -> Dict[str, Any]:
        """转换为字典格式"""
        return {
            "request_id": self.request_id,
            "response_id": self.response_id,
            "content": self.content,
            "model": self.model,
            "usage": self.usage,
            "finish_reason": self.finish_reason,
            "function_call": self.function_call,
            "tool_calls": self.tool_calls,
            "metadata": self.metadata,
            "created_at": self.created_at.isoformat()
        }


class LLMError(Exception):
    """LLM相关错误基类"""
    
    def __init__(self, message: str, error_code: Optional[str] = None, 
                 provider: Optional[str] = None, model: Optional[str] = None):
        super().__init__(message)
        self.error_code = error_code
        self.provider = provider
        self.model = model
        self.timestamp = datetime.now()


class LLMProvider(ABC):
    """LLM提供商抽象基类"""
    
    def __init__(self, name: str, api_key: Optional[str] = None, 
                 base_url: Optional[str] = None, **kwargs):
        self.name = name
        self.api_key = api_key
        self.base_url = base_url
        self.config = kwargs
        self.is_initialized = False
        self._client = None
    
    @abstractmethod
    async def initialize(self) -> None:
        """初始化提供商"""
        pass
    
    @abstractmethod
    async def cleanup(self) -> None:
        """清理资源"""
        pass
    
    @abstractmethod
    async def chat_completion(self, request: LLMRequest) -> LLMResponse:
        """发送对话补全请求"""
        pass
    
    @abstractmethod
    async def chat_completion_stream(self, request: LLMRequest) -> AsyncGenerator[str, None]:
        """流式对话补全"""
        pass
    
    @abstractmethod
    def get_available_models(self) -> List[str]:
        """获取可用模型列表"""
        pass
    
    @abstractmethod
    def get_capabilities(self) -> List[LLMCapability]:
        """获取提供商能力"""
        pass
    
    @abstractmethod
    def get_model_info(self, model: str) -> Dict[str, Any]:
        """获取模型信息"""
        pass
    
    @abstractmethod
    def validate_request(self, request: LLMRequest) -> bool:
        """验证请求是否有效"""
        pass
    
    async def health_check(self) -> bool:
        """健康检查"""
        try:
            # 发送简单的测试请求
            test_request = LLMRequest(
                messages=[{"role": "user", "content": "test"}],
                max_tokens=1
            )
            await self.chat_completion(test_request)
            return True
        except Exception as e:
            logger.error(f"Health check failed for {self.name}: {e}")
            return False
    
    def __repr__(self) -> str:
        return f"<{self.__class__.__name__}(name={self.name}, initialized={self.is_initialized})>"


class LLMProviderFactory:
    """LLM提供商工厂类"""
    
    _providers: Dict[str, type] = {}
    
    @classmethod
    def register(cls, name: str, provider_class: type):
        """注册提供商类"""
        cls._providers[name] = provider_class
        logger.info(f"Registered LLM provider: {name}")
    
    @classmethod
    def create(cls, name: str, **kwargs) -> LLMProvider:
        """创建提供商实例"""
        if name not in cls._providers:
            raise ValueError(f"Unknown LLM provider: {name}")
        
        provider_class = cls._providers[name]
        return provider_class(name=name, **kwargs)
    
    @classmethod
    def list_providers(cls) -> List[str]:
        """列出所有注册的提供商"""
        return list(cls._providers.keys()) 